<?php

namespace App\Http\Controllers\accounts;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Models\Account\Account;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherAccDetails;
use App\Models\Account\VoucherDesc;
use App\Models\Bill\BillDesc;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\DriverSettlement;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\GoodsMove\Challan;
use App\Models\Inventory\PurchaseDesc;
use App\Models\Master\Branch;
use App\Models\Master\Party;
use App\Models\Master\Vehicle;
use App\Models\Pod\Pod;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class accountController extends Controller
{

    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Accounts');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // Fetch and send all accounts
        $currentYear = $request->current_year;
        $accounts = Account::select('id', 'group_id', 'account_name','branch_id', 'created_by')
                    ->with(['group:id,group_name,is_primary'])
                    ->with('balances', function ($query) use ($currentYear) {
                        $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear)->first();
                    })
                    ->get();
        return response()->json($accounts);
    }

    public function show(Request $request, $pk)
    {
        // Fetch and send the account
        $currentYear = $request->current_year;
        $account = Account::select('id', 'group_id', 'account_name','branch_id', 'created_by')
                    ->with(['group:id,group_name,is_primary', 'balances'=> function ($query) use ($currentYear) {
                        $query->select('id','account_id','opening_bal_dr','opening_bal_cr')->where('fin_year', $currentYear)->first();
                    }])
                    ->find($pk);
          
        $account_branches = Branch::select('id','branch_code','branch_name','account_id')->whereIn('id',explode(',', $account->branch_id))->get();
          
        
        $data = [
            'id' => $account->id,
            'group' => $account->group,
            'account_name' => $account->account_name,
            'branch_id' =>  $account->branch_id,
            'opening_bal_dr' => count($account->balances) ? $account->balances[0]->opening_bal_dr : 0,
            'opening_bal_cr' => count($account->balances) ? $account->balances[0]->opening_bal_cr : 0,
            'account_branch' => $account_branches,
        ];

        return response()->json(['account' => $data]); 
    }

    public function store(Request $request)
    {
        $test = $this->is_new_permitted($request->user->id, 'Accounts');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'group' => 'required',
            'account_name' => 'required|unique:accounts,account_name',
        ], [
            'group.required' => 'Required',
            'account_name.required' => 'Required',
            'account_name.unique' => 'Already Exists!',
        ]);

        try {
            // Transaction begins
            DB::beginTransaction();

            // Save a new Account
            $account = new Account();
            $account->group_id = $request->group['id'];
            $account->account_name = $request->account_name;
            $account->account_type = $request->group['group_name'];
            $account->created_by = $request->user->id;
            $account_branches  = $request->branches;

            $account->branch_id = implode(',', $account_branches);

            $account->save();

            $bal = new Balances();
            $bal->account_id = $account->id;
            $bal->opening_bal_dr = $request->opening_bal_dr ? $request->opening_bal_dr : 0.00;
            $bal->opening_bal_cr = $request->opening_bal_cr ? $request->opening_bal_cr : 0.00;
            $bal->opening_bal_date = Carbon::now();
            $bal->fin_year = $request->current_year;
            $bal->save();

            // Save the edit log
            $newVal = $this->get_data_for_obj($account->id,$request->current_year);
            $this->createEditLog('created', 'accounts', $account->id, $account->account_name, null, $newVal, $request->user->id, $request->current_year); // Create edit log


            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Accounts');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $validatedData = $request->validate([
            'group' => 'required',
            'account_name' => 'required',
        ], [
            'group.required' => 'Required',
            'account_name.required' => 'Required',
        ]);
        
        $account = Account::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk,$request->current_year);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($account->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A Acount
            $account->group_id = $request->group['id'];
            $account->account_name = $request->account_name;
            if($account->account_type!='Party' && $account->account_type!='Staff' && $account->account_type!='Driver' && $account->account_type!='Vehicle')
            $account->account_type = $request->group['group_name'];
        
            $account->updated_by = $request->user->id;
           // Get the existing branch IDs as an array
           $account_branches = [];
           foreach ($request->account_branch as $value) {
               $account_branches[] = $value['id'];
           }
           $newly_added_branches = $request->newly_added_branches;
           $account_branches = array_merge($account_branches, $newly_added_branches);
           $account_branches = array_unique($account_branches);
           $account->branch_id = implode(',', $account_branches);
            $account->save();
            // Acount SAVED

            $bal = Balances::where('account_id',$account->id)->where('fin_year',$request->current_year)->first();
            $bal->opening_bal_dr = $request->opening_bal_dr ? $request->opening_bal_dr : 0.00;
            $bal->opening_bal_cr = $request->opening_bal_cr ? $request->opening_bal_cr : 0.00;
            $bal->save();

            $newVal = $this->get_data_for_obj($account->id,$request->current_year);
            $this->createEditLog('updated', 'accounts', $account->id, $account->account_name, $oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'Accounts');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        $account = Account::find($pk);

        if(!$account->created_by){
            return response()->json(['Pre Set accounts Cannot Be Removed'], 401);
        }

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($account->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk, $request->current_year);

        $balances = Balances::where('account_id', $pk)->delete();

        // Find and delete the account
        $account->delete();

        $this->createEditLog('deleted', 'accounts', $account->id, $account->account_name, $oldValue, null, $request->user->id,$request->current_year); // Create edit log

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $fields = ['id', 'account_name'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $query = Account::select($fields);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];
            $notInFilters = isset($request->filterby['notInFilters']) ? $request->filterby['notInFilters'] : [];
            $whereGroup = isset($request->filterby['whereGroup']) ? $request->filterby['whereGroup'] : [];
            $excludeGroup = isset($request->filterby['excludeGroup']) ? $request->filterby['excludeGroup'] : [];
            $whereInFilters = isset($request->filterby['whereIn']) ? $request->filterby['whereIn'] : [];


           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }

           foreach ($notInFilters as $notInFilter) {
               if(isset($notInFilter['key']) && isset($notInFilter['values']))
               $query->whereNotIn($notInFilter['key'], $notInFilter['values']);
           }

           foreach ($whereInFilters as $whereInFilter) {
                if(isset($whereInFilter['key']) && isset($whereInFilter['values']))
                $query->whereIn($whereInFilter['key'], $whereInFilter['values']);
           }  

           if ($whereGroup) {
                $query->where(function ($query) use ($whereGroup, $excludeGroup) {
                    $query->whereDoesntHave('group', function ($query) use ($excludeGroup) {
                        $query->whereIn('group_name', $excludeGroup);
                    })
                    ->where(function ($query) use ($whereGroup, $excludeGroup) {
                        $query->whereHas('group', function ($query) use ($whereGroup) {
                            $query->whereIn('group_name', $whereGroup);
                        })
                        ->orWhereHas('group.under_grp', function ($query) use ($whereGroup) {
                            $query->whereIn('group_name', $whereGroup);
                        });
                    });
                });
            }
        
        }
        
        $user_branch_ids = $request->user_branch_ids;

if ($user_branch_ids) {
    $query->where(function($query) use ($user_branch_ids) {
        foreach ($user_branch_ids as $branch_id) {
            $query->orWhereRaw("FIND_IN_SET(?, branch_id)", [$branch_id]);
        }
        // Add condition to check for blank or null branch_id
        $query->orWhere(function($query) {
            $query->whereNull('branch_id')
                  ->orWhere('branch_id', '');
        });
    });
}

        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('account_name', 'like', '%'. $searchVariable . '%');
        }

        

        if($request->has('with_relations')){
            $query->with($request->with_relations); 
        }

        // Limit the number of results
        $query->orderBy('account_name','asc')->take($request->limit);

        // Retrieve the filtered 
        $accounts = $query->get();
        return response(['accounts' => $accounts, 'draw' => $request->draw]);
    }

    public function autocomplete_for_bill(Request $request){
        // Initialize the query builder
        $fields = ['accounts.id', 'accounts.account_name'];
        
        // if ($request->fields) {
        //     $fields = array_merge($fields, $request->fields);
        // }
    
        // $query1 = Account::select($fields)
        // ->leftJoin('book_grs', 'accounts.id', '=', 'book_grs.acc_party_id')
        // ->whereNotNull('book_grs.acc_party_id');
        
        // $query2 = Account::select($fields)
        //     ->leftJoin('loading_slips', 'accounts.id', '=', 'loading_slips.acc_party_id')
        //     ->whereNotNull('loading_slips.acc_party_id');
        
        // // Merge the two queries using union
        // $query = $query1->union($query2)->distinct();


        // $user_branch_ids = $request->user_branch_ids;

        // if ($user_branch_ids) {
        //     $query->where(function($query) use ($user_branch_ids) {
        //         foreach ($user_branch_ids as $branch_id) {
        //             $query->orWhereRaw("FIND_IN_SET(?, branch_id)", [$branch_id]);
        //         }
        //         // Add condition to check for blank or null branch_id
        //         $query->orWhere(function($query) {
        //             $query->whereNull('branch_id')
        //                 ->orWhere('branch_id', '');
        //         });
        //     });
        // }


    
        // Check if searchVariable is provided and apply the filter
        $query = Account::select('id','account_name','account_type','branch_id')->with('party:id,party_name,account_id,party_code')->where('account_type','Party')
                ->whereHas('party', function($qr) {
                    $qr->where('account_head', 'Sundry Debtors');
                });

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('account_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->orderBy('account_name','asc')->take($request->limit);
            
        // Retrieve the filtered accounts
        $accounts = $query->get();
        return response(['accounts' => $accounts, 'draw' => $request->draw]);
    }
    

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Accounts');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;
        
        $group = $request->group;

        $currentYear = $request->current_year; // Set the current year based on your requirements

        $query = Account::select('id', 'account_name', 'account_type', 'branch_id', 'group_id','no_delete', 'created_by')
            ->with(['group:id,group_name,is_primary', 'balances'=> function ($query) use ($currentYear) {
                $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear);
            }])
            ->when($group, function ($query, $group) {
                return $query->where('group_id', $group);
            });

            $user_branch_ids = $request->user_branch_ids;

if ($user_branch_ids) {
    $query->where(function($query) use ($user_branch_ids) {
        foreach ($user_branch_ids as $branch_id) {
            $query->orWhereRaw("FIND_IN_SET(?, branch_id)", [$branch_id]);
        }
        // Add condition to check for blank or null branch_id
        $query->orWhere(function($query) {
            $query->whereNull('branch_id')
                  ->orWhere('branch_id', '');
        });
    });
}

            
            

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('account_name', 'like', $searchValue . '%');
            });
        }


        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Account::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }



    public function get_ledger(Request $request, $pk) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;

        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];
        $print_type = $request->print_type ? $request->print_type : 'Monthly';

        
        
        $account = Account::select('id', 'account_name', 'account_type', 'group_id','branch_id')
            ->with(['group:id,group_name,nature', 
            'balances'=> function ($query) use ($currentYear) {
                $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear);
            },
            'voucher_acc_details'=> function ($qr) use ($currentYear,$branch_id) {
                $qr->select('id','voucher_id','account_id','nature','dr_amount','cr_amount','active_status')
                 ->where('fin_year', $currentYear)
                 ->orderByRaw("voucher_id");

                if($branch_id){
                    $qr->whereHas('voucher', function ($subQuery) use ($branch_id,$currentYear) {
                        $subQuery->where('branch_id', $branch_id)->where('fin_year', $currentYear);
                    });
                }
            },
            'voucher_acc_details.voucher:id,branch_id,voucher_number,voucher_date,bank_date,voucher_name,t_against_id,t_against,t_against_no,narration,active_status','voucher_acc_details.account:id,account_name'])
            ->find($pk);
 
        
        $account_nature = $account->group->nature;
        
        if($account_nature=='Dr'){
            if(isset($account->balances[0])){
            $opening_balance = $account->balances[0]->opening_bal_dr - $account->balances[0]->opening_bal_cr;
            }
            else{
            $opening_balance = 0;
            }
        }
        elseif($account_nature=='Cr'){
            if(isset($account->balances[0])){
                $opening_balance = $account->balances[0]->opening_bal_cr - $account->balances[0]->opening_bal_dr;
             }
            else{
            $opening_balance = 0;
            }
        }

        // Calculate opening balance
        if ($min_date !== $from_date) {
            $opening_bal = $account->voucher_acc_details
                                ->filter(function ($voucher) use ($min_date, $from_date) {
                                    return $voucher->voucher->voucher_date >= $min_date && $voucher->voucher->voucher_date < $from_date && $voucher->active_status;
                                })
                                ->sum(function ($voucher) use ($account_nature) {
                                    if($account_nature=='Dr'){
                                       return $voucher->dr_amount - $voucher->cr_amount;
                                    }
                                    else{
                                        return $voucher->cr_amount - $voucher->dr_amount;
                                    }
                                });
                           
                         
            $opening_balance = $opening_balance+$opening_bal;
        } 
        
        
    
        // Filter voucher details by date range
        $filtered_voucher_details = $account->voucher_acc_details->filter(function ($voucher) use ($from_date, $to_date) {
            return $voucher->voucher->voucher_date >= $from_date && $voucher->voucher->voucher_date <= $to_date && $voucher->active_status;
        });


        $final_result = [];

        foreach ($filtered_voucher_details as $entry) {
            if ($entry->nature === 'dr') {
                // Find the corresponding Cr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Cr')->with(['account:id,account_name'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;

            } elseif ($entry->nature === 'cr') {
                // Find the corresponding Dr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Dr')->with(['account:id,account_name'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;
            }

            $formatted_entry = [
                'voucher_acc_details_id' => $entry->id,
                'voucher_date' => Carbon::parse($entry->voucher->voucher_date)->format('d-M-y'),
                'bank_date' => $entry->voucher->voucher_name!='TRANSACTION'? $entry->voucher->bank_date : $entry->voucher->voucher_date,
                'opposite_accounts' => $entry->opposite_accounts,
                'narration' => $entry->voucher->narration,
                'voucher_type' => $entry->voucher->voucher_name,
                'voucher_id' => $entry->voucher_id,
                'voucher_number' => $entry->voucher->voucher_number,
                'account_id' => $entry->account->id,
                'nature_of_transaction' => $entry->nature,
                'dr_amount' => $entry->dr_amount,
                'cr_amount' => $entry->cr_amount,
                't_against' => $entry->voucher->t_against,
                't_against_id' => $entry->voucher->t_against_id,
                't_against_no' => $entry->voucher->t_against_no,
            ];
        
            $final_result[] = $formatted_entry;
        }
        


        // Additional logic for handling 'Monthly' and 'Daily' print_type
            if ($print_type === 'Monthly') {
                // Group transactions by month and calculate sums
                $monthly_transactions = [];
                foreach ($final_result as $entry) {
                    $voucher_date = Carbon::parse($entry['voucher_date']);
                    $month = $voucher_date->format('M');
                    $year = $voucher_date->format('Y');
                    
                    if (!isset($monthly_transactions[$month])) {
                        $monthly_transactions[$month] = [
                            'dr_amount' => 0,
                            'cr_amount' => 0,
                        ];
                    }

                    $monthly_transactions[$month]['dr_amount'] += $entry['dr_amount'];
                    $monthly_transactions[$month]['cr_amount'] += $entry['cr_amount'];
                    $monthly_transactions[$month]['date_or_month'] = $month;
                    $monthly_transactions[$month]['account_id'] = $entry['account_id'];
                    $monthly_transactions[$month]['year'] = $year;
                }

                $data = $monthly_transactions;
            } elseif ($print_type === 'Daily') {
                // Group transactions by date and calculate sums
                $daily_transactions = [];
                foreach ($final_result as $entry) {
                    $voucher_date = Carbon::parse($entry['voucher_date']);
                    $date = $voucher_date->format('Y-m-d');
                    $formatted_date = $voucher_date->format('d-M-Y');
                    $year = $voucher_date->format('Y');
                    
                    if (!isset($daily_transactions[$date])) {
                        $daily_transactions[$date] = [
                            'dr_amount' => 0,
                            'cr_amount' => 0,
                        ];
                    }

                    $daily_transactions[$date]['dr_amount'] += $entry['dr_amount'];
                    $daily_transactions[$date]['cr_amount'] += $entry['cr_amount'];
                    $daily_transactions[$date]['date_or_month'] = $formatted_date;
                    $daily_transactions[$date]['account_id'] = $entry['account_id'];
                    $monthly_transactions[$date]['year'] = $year;
                }
                usort($daily_transactions, function($a, $b) {
                    return strtotime($a['date_or_month']) - strtotime($b['date_or_month']);
                });

                $data = $daily_transactions;
            }
            elseif ($print_type === 'All'){
                usort($final_result, function($a, $b) {
                    return strtotime($a['voucher_date']) - strtotime($b['voucher_date']);
                });
                
                $data = $final_result;
            }
        
            //$data to array
            $data = array_values($data);

        $opening_balance_dr = 0;
        $opening_balance_cr = 0;

        if($opening_balance>0 && $account_nature == 'Dr'){
            $opening_balance_dr = $opening_balance;
        }
        else if($opening_balance>0 && $account_nature == 'Cr'){
            $opening_balance_cr = $opening_balance;
        }
        else if($opening_balance<0 && $account_nature == 'Dr'){
            $opening_balance_cr = abs($opening_balance);
        }
        else if($opening_balance<0 && $account_nature == 'Cr'){
            $opening_balance_dr = abs($opening_balance);
        }else{
            $opening_balance_dr = 0;
            $opening_balance_cr = 0;
        }
    
        $datax = [
            'opening_bal_dr' => $opening_balance_dr,
            'opening_bal_cr' => $opening_balance_cr,
            'listItems' =>  $data,
            'account_nature' => $account_nature,
        ];
    
        return response()->json($datax);
    }

    public function get_bank_reconcile(Request $request, $pk) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;

        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];
        $print_type = $request->print_type ? $request->print_type : 'Monthly';

        
        
        $account = Account::select('id', 'account_name', 'account_type', 'group_id','branch_id')
            ->with(['group:id,group_name,nature', 
            'balances'=> function ($query) use ($currentYear) {
                $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear);
            },
            'voucher_acc_details'=> function ($qr) use ($currentYear,$branch_id) {
                $qr->select('id','voucher_id','account_id','nature','dr_amount','cr_amount','active_status')
                 ->where('fin_year', $currentYear)
                 ->orderByRaw("voucher_id");

                if($branch_id){
                    $qr->whereHas('voucher', function ($subQuery) use ($branch_id,$currentYear) {
                        $subQuery->where('branch_id', $branch_id)->where('fin_year', $currentYear);
                    });
                }
            },
            'voucher_acc_details.voucher:id,branch_id,voucher_number,voucher_date,bank_date,voucher_name,t_against_id,t_against,t_against_no,narration,active_status','voucher_acc_details.account:id,account_name'])
            ->find($pk);
 
        
        $account_nature = $account->group->nature;
        
        if($account_nature=='Dr'){
            if(isset($account->balances[0])){
            $opening_balance = $account->balances[0]->opening_bal_dr - $account->balances[0]->opening_bal_cr;
            }
            else{
            $opening_balance = 0;
            }
        }
        elseif($account_nature=='Cr'){
            if(isset($account->balances[0])){
                $opening_balance = $account->balances[0]->opening_bal_cr - $account->balances[0]->opening_bal_dr;
             }
            else{
            $opening_balance = 0;
            }
        }

        $opening_bal_bank = $account->voucher_acc_details
                                ->filter(function ($voucher) use ($min_date, $from_date) {
                                    return $voucher->voucher->bank_date && $voucher->voucher->voucher_date >= $min_date && $voucher->voucher->voucher_date < $from_date && $voucher->active_status;
                                })
                                ->sum(function ($voucher) use ($account_nature) {
                                    if($account_nature=='Dr'){
                                       return $voucher->dr_amount - $voucher->cr_amount;
                                    }
                                    else{
                                        return $voucher->cr_amount - $voucher->dr_amount;
                                    }
                                });           
        $opening_bal_bank = $opening_bal_bank+$opening_balance ;

        // Calculate opening balance
        if ($min_date !== $from_date) {
            $opening_bal = $account->voucher_acc_details
                                ->filter(function ($voucher) use ($min_date, $from_date) {
                                    return $voucher->voucher->voucher_date >= $min_date && $voucher->voucher->voucher_date < $from_date && $voucher->active_status;
                                })
                                ->sum(function ($voucher) use ($account_nature) {
                                    if($account_nature=='Dr'){
                                       return $voucher->dr_amount - $voucher->cr_amount;
                                    }
                                    else{
                                        return $voucher->cr_amount - $voucher->dr_amount;
                                    }
                                });

            $opening_balance = $opening_balance+$opening_bal;


            
        } 


        
        
    
        // Filter voucher details by date range
        $filtered_voucher_details = $account->voucher_acc_details->filter(function ($voucher) use ($from_date, $to_date) {
            return $voucher->voucher->voucher_date >= $from_date && $voucher->voucher->voucher_date <= $to_date && $voucher->active_status;
        });


        $final_result = [];

        foreach ($filtered_voucher_details as $entry) {
            if ($entry->nature === 'dr') {
                // Find the corresponding Cr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Cr')->with(['account:id,account_name'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;

            } elseif ($entry->nature === 'cr') {
                // Find the corresponding Dr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Dr')->with(['account:id,account_name'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;
            }

            $formatted_entry = [
                'voucher_acc_details_id' => $entry->id,
                'voucher_date' => Carbon::parse($entry->voucher->voucher_date)->format('d-M-y'),
                'bank_date' => $entry->voucher->voucher_name!='TRANSACTION'? $entry->voucher->bank_date : $entry->voucher->voucher_date,
                'opposite_accounts' => $entry->opposite_accounts,
                'narration' => $entry->voucher->narration,
                'voucher_type' => $entry->voucher->voucher_name,
                'voucher_id' => $entry->voucher_id,
                'voucher_number' => $entry->voucher->voucher_number,
                'account_id' => $entry->account->id,
                'nature_of_transaction' => $entry->nature,
                'dr_amount' => $entry->dr_amount,
                'cr_amount' => $entry->cr_amount,
                't_against' => $entry->voucher->t_against,
                't_against_id' => $entry->voucher->t_against_id,
                't_against_no' => $entry->voucher->t_against_no,
            ];
        
            $final_result[] = $formatted_entry;
        }
        


        // Additional logic for handling 'Monthly' and 'Daily' print_type
            if ($print_type === 'Monthly') {
                // Group transactions by month and calculate sums
                $monthly_transactions = [];
                foreach ($final_result as $entry) {
                    $voucher_date = Carbon::parse($entry['voucher_date']);
                    $month = $voucher_date->format('M');
                    $year = $voucher_date->format('Y');
                    
                    if (!isset($monthly_transactions[$month])) {
                        $monthly_transactions[$month] = [
                            'dr_amount' => 0,
                            'cr_amount' => 0,
                        ];
                    }

                    $monthly_transactions[$month]['dr_amount'] += $entry['dr_amount'];
                    $monthly_transactions[$month]['cr_amount'] += $entry['cr_amount'];
                    $monthly_transactions[$month]['date_or_month'] = $month;
                    $monthly_transactions[$month]['account_id'] = $entry['account_id'];
                    $monthly_transactions[$month]['year'] = $year;
                }

                $data = $monthly_transactions;
            } elseif ($print_type === 'Daily') {
                // Group transactions by date and calculate sums
                $daily_transactions = [];
                foreach ($final_result as $entry) {
                    $voucher_date = Carbon::parse($entry['voucher_date']);
                    $date = $voucher_date->format('Y-m-d');
                    $formatted_date = $voucher_date->format('d-M-Y');
                    $year = $voucher_date->format('Y');
                    
                    if (!isset($daily_transactions[$date])) {
                        $daily_transactions[$date] = [
                            'dr_amount' => 0,
                            'cr_amount' => 0,
                        ];
                    }

                    $daily_transactions[$date]['dr_amount'] += $entry['dr_amount'];
                    $daily_transactions[$date]['cr_amount'] += $entry['cr_amount'];
                    $daily_transactions[$date]['date_or_month'] = $formatted_date;
                    $daily_transactions[$date]['account_id'] = $entry['account_id'];
                    $monthly_transactions[$date]['year'] = $year;
                }
                usort($daily_transactions, function($a, $b) {
                    return strtotime($a['date_or_month']) - strtotime($b['date_or_month']);
                });

                $data = $daily_transactions;
            }
            elseif ($print_type === 'All'){
                usort($final_result, function($a, $b) {
                    return strtotime($a['voucher_date']) - strtotime($b['voucher_date']);
                });
                
                $data = $final_result;
            }
        
            //$data to array
            $data = array_values($data);

        $opening_balance_dr = 0;
        $opening_balance_cr = 0;

        if($opening_balance>0 && $account_nature == 'Dr'){
            $opening_balance_dr = $opening_balance;
        }
        else if($opening_balance>0 && $account_nature == 'Cr'){
            $opening_balance_cr = $opening_balance;
        }
        else if($opening_balance<0 && $account_nature == 'Dr'){
            $opening_balance_cr = abs($opening_balance);
        }
        else if($opening_balance<0 && $account_nature == 'Cr'){
            $opening_balance_dr = abs($opening_balance);
        }else{
            $opening_balance_dr = 0;
            $opening_balance_cr = 0;
        }

        $opening_bal_bank_dr = 0;
        $opening_bal_bank_cr = 0;

        if($opening_bal_bank>0 && $account_nature == 'Dr'){
            $opening_bal_bank_dr = $opening_bal_bank;
        }
        else if($opening_bal_bank>0 && $account_nature == 'Cr'){
            $opening_bal_bank_cr = $opening_bal_bank;
        }
        else if($opening_bal_bank<0 && $account_nature == 'Dr'){
            $opening_bal_bank_cr = abs($opening_bal_bank);
        }
        else if($opening_bal_bank<0 && $account_nature == 'Cr'){
            $opening_bal_bank_dr = abs($opening_bal_bank);
        }else{
            $opening_bal_bank_dr = 0;
            $opening_bal_bank_cr = 0;
        }

    
        $datax = [
            'opening_bal_dr' => $opening_balance_dr,
            'opening_bal_cr' => $opening_balance_cr,
            'opening_bal_bank_dr' => $opening_bal_bank_dr,
            'opening_bal_bank_cr' => $opening_bal_bank_cr,
            
            'listItems' =>  $data,
            'account_nature' => $account_nature,
        ];
    
        return response()->json($datax);
    }


    public function get_tds_report(Request $request){
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->report_data['branch_id'];

        $from_date = $request->report_data['from_date'] ? $request->report_data['from_date'] : $min_max_dates['min_date'] ;
        $to_date = $request->report_data['to_date'] ? $request->report_data['to_date'] : $min_max_dates['max_date'];

        $pk = $this->get_acc_id($request->report_data['tds_type']);

        $nature =$request->report_data['tds_type'] == 'TDS Payable' ? 'cr' : 'dr';
        
        
        $account = Account::select('id', 'account_name', 'account_type', 'group_id')
            ->with(['group:id,group_name,nature', 
            'balances'=> function ($query) use ($currentYear) {
                $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear);
            },
            'voucher_acc_details'=> function ($qr) use ($currentYear,$branch_id,$nature) {
                $qr->select('id','voucher_id','account_id','nature','dr_amount','cr_amount','active_status')
                 ->where('fin_year', $currentYear)
                 ->where('nature',$nature)
                 ->orderByRaw("voucher_id");

                if($branch_id){
                    $qr->whereHas('voucher', function ($subQuery) use ($branch_id,$currentYear) {
                        $subQuery->where('branch_id', $branch_id)->where('fin_year', $currentYear);
                    });
                }
            },
            'voucher_acc_details.voucher:id,branch_id,voucher_number,voucher_date,voucher_name,t_against_id,t_against,t_against_no,narration,active_status','voucher_acc_details.account:id,account_name'])
            ->find($pk);
 
        
       
    
        // Filter voucher details by date range
        $filtered_voucher_details = $account->voucher_acc_details->filter(function ($voucher) use ($from_date, $to_date) {
            return $voucher->voucher->voucher_date >= $from_date && $voucher->voucher->voucher_date <= $to_date && $voucher->active_status;
        });


        $final_result = [];

        foreach ($filtered_voucher_details as $entry) {
            $truck_no = '';
            $freight = '';
            $percent = '';
            $amount = '';
            $pan_number = '';

            if ($entry->nature === 'dr') {
                // Find the corresponding Cr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Cr')->with(['account:id,account_name','account.party:id,party_name,account_id,pan_number'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;

                if($entry->voucher->t_against=='LOADING SLIP'){

                        $loading_slip = LoadingSlip::select('id','ls_number','source_id','destination_id','vehicle_id','total_without_gst','tds_percent','tds_amount','acc_party_id')
                                ->with(['source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number','acc_party:id,account_name','acc_party.party:id,party_name,account_id,pan_number'])
                                ->find($entry->voucher->t_against_id);

                        $copy = json_decode(json_encode($loading_slip));
                        $truck_no = $loading_slip->vehicle?$loading_slip->vehicle->vehicle_number:'';
                        $freight = $loading_slip->total_without_gst;
                        $percent = $loading_slip->tds_percent;
                        $amount = $loading_slip->tds_amount;
                        $pan_number = $copy->acc_party?$copy->acc_party->party->pan_number:'';
                
                }

                if($entry->voucher->voucher_name=='Receipt Voucher'){

                    
                    $bill = VoucherDesc::with(['bill:id,bill_number,total_without_gst'])
                                ->where('voucher_id', $entry->voucher->id)->first();



                    if($bill->bill){
                        $freight = $bill->bill->total_without_gst;
                        $percent = round(($entry->dr_amount / $bill->bill->total_without_gst ) * 100,2) ;
                        $amount =  $entry->dr_amount;
                    }

                    $pan_number = $entry->opposite_accounts[0]->account->party ? $entry->opposite_accounts[0]->account->party->pan_number : '';

                }


            } elseif ($entry->nature === 'cr') {
                // Find the corresponding Dr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Dr')->with(['account:id,account_name','account.party:id,party_name,account_id,pan_number'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;

                

                if($entry->voucher->t_against=='CHALLAN'){

                        $challan = Challan::select('id','challan_number','source_id','destination_id','vehicle_id', 'lh_net_freight','lh_tds_percent','lh_tds','lorry_hire_from')
                                ->with(['source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number', 'lorry_hire_from:id,account_name', 'lorry_hire_from.party:id,account_id,pan_number'])
                                ->find($entry->voucher->t_against_id);
                        
                        $copy = json_decode(json_encode($challan));
                        $truck_no = $challan->vehicle?$challan->vehicle->vehicle_number:'';
                        $freight = $challan->lh_net_freight;
                        $percent = $challan->lh_tds_percent;
                        $amount = $challan->lh_tds;
                        $pan_number = $copy->lorry_hire_from?$copy->lorry_hire_from->party->pan_number:'';
                    
                }


                if($entry->voucher->voucher_name=='Payment Voucher'){

                    $challan = VoucherDesc::with(['challan:id,challan_number,source_id,destination_id,vehicle_id,lh_net_freight,lh_tds_percent,lh_tds,lorry_hire_from'])
                    ->where('voucher_id', $entry->voucher->id)->first();

                                if($challan->challan){
                                    $truck_no = $challan->challan->vehicle?$challan->challan->vehicle->vehicle_number:'';
                                    $freight = $challan->challan->lh_net_freight;
                                    $percent = round(($entry->cr_amount / $challan->challan->lh_net_freight ) * 100,2) ;
                                    $amount =  $entry->cr_amount;
                                }

                    
                    $pan_number = $entry->opposite_accounts[0]->account->party ? $entry->opposite_accounts[0]->account->party->pan_number : '';
                
                }

                


            }

            $formatted_entry = [
                'voucher_acc_details_id' => $entry->id,
                'voucher_date' => Carbon::parse($entry->voucher->voucher_date)->format('d-M-y'),
                'opposite_accounts' => $entry->opposite_accounts,
                'narration' => $entry->voucher->narration,
                'voucher_type' => $entry->voucher->voucher_name,
                'voucher_id' => $entry->voucher_id,
                'voucher_number' => $entry->voucher->voucher_number,
                'account_id' => $entry->account->id,
                'nature_of_transaction' => $entry->nature,
                'dr_amount' => $entry->dr_amount,
                'cr_amount' => $entry->cr_amount,
                't_against' => $entry->voucher->t_against,
                't_against_id' => $entry->voucher->t_against_id,
                't_against_no' => $entry->voucher->t_against_no,

                'truck_no' => $truck_no,
                'freight' => $freight,
                'percent' => $percent,
                'amount' => $amount,
                'pan_number' => $pan_number,

            ];
        
            $final_result[] = $formatted_entry;
        }
        


        usort($final_result, function($a, $b) {
            return strtotime($a['voucher_date']) - strtotime($b['voucher_date']);
        });
        $data = $final_result;
        
            //$data to array
            $data = array_values($data);

        $datax = [
            'listItems' =>  $data,
        ];
    
        return response()->json($datax);
    }

    public function get_trial_balance(Request $request) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;
        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];

        $group_id = $request->group_id;
    
        $groups = Group::select('id', 'group_name', 'nature')
                    ->when(!$group_id, function ($query) {
                        return $query->where('is_primary', 1);
                    })
                    ->when($group_id, function ($query, $group_id) {
                        return $query->where('id', $group_id);
                    })
                    ->get();

    
        $data = [];
        
        $sum_of_opening_dr = 0;
        $sum_of_opening_cr = 0;
    
        foreach ($groups as $group) {
            $under_group = Group::select('id', 'group_name', 'nature')->where('under_grp_id', $group->id)->get();
            $ug_ids = $under_group->pluck('id');
    
            $accounts = Account::select('id')->where('group_id', $group->id)
                ->orWhereIn('group_id', $ug_ids)
                ->pluck('id');
    
            $dr_total = 0;
            $cr_total = 0;
    
            foreach ($accounts as $account_id) {
                $opening_balance_dr = 0;
                $opening_balance_cr = 0;
    
                $balance = $this->calculate_balance_for_account($account_id, $min_date,$from_date,$to_date,$currentYear, $branch_id);
                
                if($balance['nature_of_op_bal'] == 'Dr'){
                    $opening_balance_dr = $balance['opening_balance'];
                }
                else if($balance['nature_of_op_bal'] == 'Cr'){
                    $opening_balance_cr = $balance['opening_balance'];
                }


                if($balance['nature_of_balance']=='Dr'){
                    $dr_total += $balance['current_balance'];
                }
                else{
                    $cr_total += $balance['current_balance'];
                }

                $sum_of_opening_dr += $opening_balance_dr;
                $sum_of_opening_cr += $opening_balance_cr;
            }
    
            $diff = $dr_total - $cr_total;
            if($diff>=0 ){
                $dr_balance = $diff;
                $cr_balance = 0;
            }
            else if($diff<0){
                $dr_balance = 0;
                $cr_balance = abs($diff);
            } 
    
            if($dr_balance>0 || $cr_balance>0){
                $data[$group->group_name] = [
                    'group' => $group,
                    'opening_dr' => 0,
                    'opening_cr' => 0,
                    'total_dr_amount' => 0,
                    'total_cr_amount' => 0,
                    
                    'dr_balance' => $dr_balance,
                    'cr_balance' => $cr_balance,
                ];

            }
            
        }
    
        $data = array_values($data);
        $opening_difference_dr = max($sum_of_opening_cr - $sum_of_opening_dr, 0);
        $opening_difference_cr = max($sum_of_opening_dr - $sum_of_opening_cr, 0);
    
        $datax = [
            'opening_difference_dr' => $opening_difference_dr,
            'opening_difference_cr' => $opening_difference_cr,
            'listItems' => $data,
        ];
    
        return response()->json($datax);
    }

    public function get_truck_pl_ledger(Request $request, $pk) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;

        $truck_id = $pk;

        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];
        $print_type = $request->print_type ? $request->print_type : 'Monthly';

        $payment_voucher = VoucherDesc::select('id','voucher_id','voucher_number','receipt_amt','v_acc_det_id')->where('against', 'VEHICLE_COST_CENTER')
                            ->where('against_id', $truck_id)
                            ->whereIn('voucher_name', ['Payment Voucher', 'Journal'])
                            ->where('fin_year', $currentYear)
                            ->with(['voucher:id,voucher_number,voucher_date,narration','voucher_acc_details:id,account_id','voucher_acc_details.account:id,account_name'])
                            ->whereHas('voucher', function ($query) use ($from_date, $to_date,$branch_id) {
                                $query->where('voucher_date', '>=', $from_date)
                                    ->where('voucher_date', '<=', $to_date)
                                    ->when($branch_id, function ($query, $branch_id) {
                                        return $query->where('branch_id', $branch_id);
                                    });
                            })
                            ->get();

       
        $diesel_receipt = DieselReceipt::select('id','dr_number','booking_date','amount','remarks','dr_on_account')
                            ->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('booking_date', '>=', $from_date)
                            ->where('booking_date', '<=', $to_date)
                            ->with(['dr_on_account:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();

        $driver_settlement_expense = DriverSettlement::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('settle_date', '>=', $from_date)
                            ->where('settle_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->where('driver_expense', '>=', 0)
                            ->get();
        
        $driver_settlement_income = DriverSettlement::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('settle_date', '>=', $from_date)
                            ->where('settle_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->where('driver_expense', '<', 0)
                            ->get();

        $purchase_desc = PurchaseDesc::select('id','bill_id','bill_number','net_amount','dr_account_id','vehicle_id','product_id')
                            ->where('vehicle_id',$truck_id)
                            ->whereHas('purchase', function ($query) use ($from_date, $to_date,$branch_id) {
                                $query->where('bill_date', '>=', $from_date)
                                    ->where('bill_date', '<=', $to_date)
                                    ->where('bill_name', '!=', 'Sale')
                                    ->when($branch_id, function ($query, $branch_id) {
                                        return $query->where('branch_id', $branch_id);
                                    });
                            })
                            ->with(['dr_account:id,account_name','purchase:id,bill_name,bill_number,bill_date,remarks','product:id,product_name'])
                           ->where('fin_year',$currentYear)
                           ->get();
        
    
        $bill_desc = BillDesc::where('vehicle_id',$truck_id)
                           ->whereHas('bill', function ($query) use ($from_date, $to_date,$branch_id) {
                               $query->where('bill_date', '>=', $from_date)
                                   ->where('bill_date', '<=', $to_date)
                                   ->when($branch_id, function ($query, $branch_id) {
                                       return $query->where('branch_id', $branch_id);
                                   });
                           })
                           ->with(['bill:id,bill_number,bill_date,remarks,income','bill.income:id,account_name','vehicle:id,vehicle_number'])
                          ->where('fin_year',$currentYear)
                          ->get();
                        
        $pods = Pod::select('id','pod_number','delivery_date','grand_total','remarks','income_account_id')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('delivery_date', '>=', $from_date)
                            ->where('delivery_date', '<=', $to_date)
                            ->with(['income:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();
                        
        $gr_without_pod = BookGr::select('id','gr_number','gr_date','grand_total','remarks','income_account_id')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->with(['income:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->whereDoesntHave('pod_desc')
                            ->get();
        
        $lorry_hire_expense = BookGr::select('id','gr_number','gr_date','dr_on_account','lh_net_freight')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->where('lh_net_freight','>',0)
                            ->with(['dr_on_account:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();
                        
        $loading_slip = LoadingSlip::select('id','ls_number','ls_date','branch_id','total_without_gst','remarks','income')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('ls_date', '>=', $from_date)
                            ->where('ls_date', '<=', $to_date)
                            ->with(['income:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();
            
        $challan = Challan::select('id','challan_number','challan_date','vehicle_id','dr_on_account','lh_net_freight','remarks')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('challan_date', '>=', $from_date)
                            ->where('challan_date', '<=', $to_date)
                            ->with(['dr_on_account:id,account_name'])
                            ->where('lh_net_freight','>',0)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();
   
        $driver_advance = BookGr::with(['driver_payment_from:id,account_name'])->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->where('driver_payment_amt', '>', 0)
                            ->get();
        
        
        // LOOPING ALL THE ABOVE
        $final_result = [];
        

        // Combine results into a single array
        foreach ($payment_voucher as $item) {
            $final_result[] = [
                'date' => $item->voucher->voucher_date,
                'remarks' => $item->voucher->narration,
                'on_account' => $item->voucher_acc_details->account, // Set the value based on your logic
                'against' => 'Voucher', // Set the value based on your logic
                'against_id' => $item->voucher_id,
                'against_no' => $item->voucher_number,
                'transaction_type' => 'Voucher', // Set the value based on your logic
                'dr_amount' => $item->receipt_amt,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($diesel_receipt as $item) {
            $hehe = $item->toArray();
            $drOnAccount = $hehe['dr_on_account'];

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->booking_date,
                'remarks' => $item->remarks,
                'on_account' => $drOnAccount, // Set the value based on your logic
                'against' => 'Diesel Receipt', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->dr_number,
                'transaction_type' => 'Diesel Receipt', // Set the value based on your logic
                'dr_amount' => $item->amount,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($driver_advance as $item) {
            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->gr_date,
                'remarks' => 'Driver Adv for '.$item->gr_number,
                'on_account' => $item->driver_payment_from, // Set the value based on your logic
                'against' => 'GR', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->gr_number,
                'transaction_type' => 'GR', // Set the value based on your logic
                'dr_amount' => $item->driver_payment_amt,
                'cr_amount' => '', // Set the value based on your logic
            ];
        }

        foreach ($lorry_hire_expense as $item) {
            $hehe = $item->toArray();
            $drOnAccount = $hehe['dr_on_account'];

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->gr_date,
                'remarks' => 'Lorry Hired',
                'on_account' => $drOnAccount, // Set the value based on your logic
                'against' => 'Lorry Hire Expense', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->gr_number,
                'transaction_type' => 'GR', // Set the value based on your logic
                'dr_amount' => $item->lh_net_freight,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($challan as $item) {
            $hehe = $item->toArray();
            $drOnAccount = $hehe['dr_on_account'];

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->challan_date,
                'remarks' => 'Lorry Hired',
                'on_account' => $drOnAccount, // Set the value based on your logic
                'against' => 'Lorry Hire Expense', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->gr_number,
                'transaction_type' => 'Challan', // Set the value based on your logic
                'dr_amount' => $item->lh_net_freight,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($driver_settlement_expense as $item) {
            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->settle_date,
                'remarks' => $item->remarks,
                'on_account' => ['id'=>19, 'account_name'=>'Trip Expense'] , // Set the value based on your logic
                'against' => 'Driver Settlement', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->ds_number,
                'transaction_type' => 'Driver Settlement', // Set the value based on your logic
                'dr_amount' => $item->driver_expense,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($driver_settlement_income as $item) {
            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->settle_date,
                'remarks' => $item->remarks,
                'on_account' =>  ['id'=>19, 'account_name'=>'Trip Expense'], // Set the value based on your logic
                'against' => 'Driver Settlement', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->ds_number,
                'transaction_type' => 'Driver Settlement', // Set the value based on your logic
                'dr_amount' => 0,
                'cr_amount' => abs($item->driver_expense), // Set the value based on your logic
            ];
        }

        foreach ($purchase_desc as $item) {

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->purchase->bill_date,
                'remarks' => $item->purchase->remarks,
                'on_account' => $item->dr_account, // Se the value based on your logic
                'against' => $item->purchase->bill_name, // Set the value based on your logic
                'against_id' => $item->bill_id,
                'against_no' => $item->bill_number,
                'transaction_type' => $item->purchase->bill_name, // Set the value based on your logic
                'dr_amount' => $item->net_amount,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($bill_desc as $item) {
            $hehe = $item->toArray();
            $cr_on_acc = $hehe['bill']['income'];

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->bill->bill_date,
                'remarks' => $item->bill->remarks,
                'on_account' => $cr_on_acc, // Se the value based on your logic
                'against' => 'Bill', // Set the value based on your logic
                'against_id' => $item->bill_id,
                'against_no' => $item->bill_number,
                'transaction_type' => 'Bill', // Set the value based on your logic
                'dr_amount' => 0,
                'cr_amount' => $item->grand_total, // Set the value based on your logic
            ];
        }

        foreach ($pods as $item) {
            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->delivery_date,
                'remarks' => $item->remarks,
                'on_account' => $item->income, // Set the value based on your logic
                'against' => 'POD', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->pod_number,
                'transaction_type' => 'POD', // Set the value based on your logic
                'dr_amount' => '',
                'cr_amount' => $item->grand_total, // Set the value based on your logic
            ];
        }

        foreach ($gr_without_pod as $item) {
            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->gr_date,
                'remarks' => $item->remarks,
                'on_account' => $item->income, // Set the value based on your logic
                'against' => 'GR', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->gr_number,
                'transaction_type' => 'GR', // Set the value based on your logic
                'dr_amount' => '',
                'cr_amount' => $item->grand_total, // Set the value based on your logic
            ];
        }

        foreach ($loading_slip as $item) {
            $hehe = $item->toArray();
            $income = $hehe['income'];

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->ls_date,
                'remarks' => $item->remarks,
                'on_account' => $income, // Set the value based on your logic
                'against' => 'Loading Slip', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->ls_number,
                'transaction_type' => 'Loading Slip', // Set the value based on your logic
                'dr_amount' => '',
                'cr_amount' => $item->total_without_gst, // Set the value based on your logic
            ];
        }
                

        // Sort final_result based on 'date'.
        usort($final_result, function ($a, $b) {
            return strtotime($a['date']) - strtotime($b['date']);
        });




    
        return response()->json($final_result);
    }

    public function get_truck_pl_description(Request $request) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;
        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];

        if($request->truck_id){
            $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->where('id',$request->truck_id)->get();
        }
        else{
            $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->get();
        }

        $data = [];

        foreach ($trucks as $truck) {
            $truck_id = $truck->id;
            $truck_no = $truck->vehicle_number;

            $truck_data = $this->get_truck_data_with_sums($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, $branch_id);
            
            $truck_data['account_id'] = $truck['account_id'];

            if($truck_data['dr_amount']>0 || $truck_data['cr_amount']>0){
                $data[] = $truck_data;
            }
        }


        return response()->json($data);

    }

    public function get_truck_ledger_balance(Request $request) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;
        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];

        if($request->truck_id){
            $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->where('id',$request->truck_id)->get();
        }
        else{
            $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->get();
        }

        

        $data = [];

        foreach ($trucks as $truck) {
            $truck_id = $truck->id;
            $truck_no = $truck->vehicle_number;

            $truck_data = $this->get_truck_data($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, $branch_id);
            
            $truck_data['account_id'] = $truck['account_id'];

            if($truck_data['dr_amount']>0 || $truck_data['cr_amount']>0){
                $data[] = $truck_data;
            }
        }


        return response()->json($data);
    }

    public function get_expanded_group(Request $request, $pk) {

        $currentYear = $request->current_year;

        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;
        $min_date = $request->min_date ? $request->min_date : $min_max_dates->min_date ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates->min_date ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates->max_date;


    
        $group = Group::select('id', 'group_name', 'nature', 'group_type')->find($pk);
    
            $under_group = Group::select('id', 'group_name', 'nature')->where('under_grp_id', $group->id)->get();
            $ug_ids = $under_group->pluck('id');
    
            $accounts = Account::select('id','account_name','group_id')->where('group_id', $group->id)
                        ->orWhereIn('group_id', $ug_ids)
                        ->get();
    
            $group_total_dr = 0;
            $group_total_cr = 0;
            $opening_group_total_dr =0 ;
            $opening_group_total_cr =0;

    
            foreach ($accounts as $account) {
                $opening_dr = 0;
                $opening_cr = 0;
                $dr_balance = 0;
                $cr_balance = 0;

                $total_dr_amount = 0;
                $total_cr_amount = 0;

                $balance = $this->calculate_balance_for_account($account->id, $min_date,$from_date,$to_date,$currentYear,$branch_id);
                

                $total_dr_amount = $balance['totalDrAmount'];
                $total_cr_amount = $balance['totalCrAmount'];

                if($balance['nature_of_op_bal']=='Dr'){
                    $opening_dr = $balance['opening_balance'];
                    $opening_group_total_dr += $opening_dr;
                }
                elseif($balance['nature_of_op_bal']=='Cr'){
                    $opening_cr = $balance['opening_balance'];
                    $opening_group_total_cr += $opening_cr;  
                }


                if($balance['nature_of_balance']=='Dr'){
                    $dr_balance = $balance['current_balance'];
                    $group_total_dr += $dr_balance;
                }
                elseif($balance['nature_of_balance']=='Cr'){
                    $cr_balance = $balance['current_balance'];
                    $group_total_cr += $cr_balance;  
                }
                  
                // Check if $account->group_id inside $ug_ids
                $is_under_group = $ug_ids->contains($account->group_id);
                $sub_group = $is_under_group ? $account->group->group_name : $group->group_name;
    
                if($dr_balance>0 || $cr_balance>0){

                    $account_data[$sub_group][] = [
                        'account' => $account,
                        'opening_dr' => $opening_dr,
                        'opening_cr' => $opening_cr,

                        'total_dr_amount' => $total_dr_amount>0?$total_dr_amount:'',
                        'total_cr_amount' => $total_cr_amount>0?$total_cr_amount:'',
                        
                        'dr_balance' => $dr_balance,
                        'cr_balance' => $cr_balance,
                    ];


                    $sg_tot_dr = $account_data[$sub_group]['sub_group_total_dr'] ?? 0;
                    $sg_tot_cr = $account_data[$sub_group]['sub_group_total_cr'] ?? 0;
                    $account_data[$sub_group]['sub_group_total_dr'] = $sg_tot_dr + $dr_balance;
                    $account_data[$sub_group]['sub_group_total_cr'] = $sg_tot_cr + $cr_balance;

                    $tot_sg_tot_dr = $account_data[$sub_group]['sub_group_total_dr_amount'] ?? 0;
                    $tot_sg_tot_cr = $account_data[$sub_group]['sub_group_total_cr_amount'] ?? 0;
                    $account_data[$sub_group]['sub_group_total_dr_amount'] = $tot_sg_tot_dr + $total_dr_amount;
                    $account_data[$sub_group]['sub_group_total_cr_amount'] = $tot_sg_tot_cr + $total_cr_amount;


                    $op_sg_tot_dr = $account_data[$sub_group]['sub_group_total_opening_dr'] ?? 0;
                    $op_sg_tot_cr = $account_data[$sub_group]['sub_group_total_opening_cr'] ?? 0;
                    $account_data[$sub_group]['sub_group_total_opening_dr'] = $op_sg_tot_dr + $opening_dr;
                    $account_data[$sub_group]['sub_group_total_opening_cr'] = $op_sg_tot_cr + $opening_cr;

                }
                                
            }
    

            $group_data = [
                'group_total_dr' => $group_total_dr,
                'group_total_cr' => $group_total_cr,
                'opening_group_total_cr' => $opening_group_total_cr,
                'opening_group_total_dr' => $opening_group_total_dr,
                'group_name' => $group->group_name,
                'listItems' => $account_data??null,
            ];
    
        return response()->json($group_data);
    }

    public function get_profit_and_loss(Request $request){
        
        $currentYear = $request->current_year;


        $min_max_dates = $this->get_min_max_date($currentYear);


        $primary_groups = Group::select('id', 'group_name', 'nature', 'group_type')->where('is_primary', 1)->whereIn('group_type',['Income','Expense'])->get();

        $expense_side = [];
        $income_side = [];

        foreach ($primary_groups as $primary_grp) {
            
            $data = $this->get_expanded_group($request, $primary_grp->id)->original;

            
            if($primary_grp->group_type == 'Expense'){
                if(isset($data['listItems'])){
                    $expense_side[] = $data;
                }
            }
            else if($primary_grp->group_type == 'Income'){
                if(isset($data['listItems'])){
                    $income_side[] = $data;
                }
            }
        

        }

        return response()->json(['expense_side' => $expense_side, 'income_side' => $income_side]);
        

    }

    public function get_balance_sheet(Request $request){
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $primary_groups = Group::select('id', 'group_name', 'nature', 'group_type')->where('is_primary', 1)->whereIn('group_type',['Asset','Liability'])->get();

        $asset_side = [];
        $liability_side = [];

        foreach ($primary_groups as $primary_grp) {
            
            $data = $this->get_expanded_group($request, $primary_grp->id)->original;
            
            if($primary_grp->group_type == 'Asset'){
                if(isset($data['listItems'])){
                    $asset_side[] = $data;
                }
            }
            else if($primary_grp->group_type == 'Liability'){
                if(isset($data['listItems'])){
                    $liability_side[] = $data;
                }
            }
        }

        // Calculate the total of asset_side and liability_side
        $profit_and_loss_amount = $this->get_profit_and_loss_amount($request);
        
        if($profit_and_loss_amount['profit_and_loss']>=0){
            $liability_side[] = [
                'group_total_dr' => 0,
                'group_total_cr' => $profit_and_loss_amount['profit_and_loss'],
                'group_name' => 'Net Profit',
                'listItems' => null,
            ];
        }
        else{
            $asset_side[] = [
                'group_total_dr' => abs($profit_and_loss_amount['profit_and_loss']),
                'group_total_cr' => 0,
                'group_name' => 'Net Loss',
                'listItems' => null,
            ];
        }

        return response()->json(['asset_side' => $asset_side, 'liability_side' => $liability_side]);
        

    }


    public function get_balance_for_account(Request $request, $pk){
        $balance = $this->calculate_balance_for_account($pk,$request->min_date,$request->from_date,$request->to_date,$request->current_year,null);
        return response()->json($balance);
    }

    public function update_balance_of_account(Request $request){
        $res = $this->update_balance_for_account($request->current_year, $request->account_ids);
        return response()->json("Success");
    }

    public function update_openings(Request $request){

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

        $account_ids = Account::whereNotNull('group_id')
                        ->whereHas('group', function($qr) {
                            $qr->whereIn('group_type', ['Asset', 'Liability']);
                        })
                        ->pluck('id')
                        ->toArray();
        $this->update_balance_for_account($request->current_year, $account_ids);

        // Update the opening balance of profit and loss account
        $allowed_fin_years = explode(',', config('app.ALLOWED_FIN_YEAR'));

        $sum_of_pl = 0;

        $index = array_search($request->current_year, $allowed_fin_years);
        if ($index !== false) {
            // Get an array from the index and so on
            $update_fin_years = array_slice($allowed_fin_years, $index);

             // Loop through the array and update the balances OF P&L
             for ($i = 0; $i < count($update_fin_years); $i++) {
                $fin_year = $update_fin_years[$i];
    
                $min_max_date = $this->get_min_max_date($fin_year);
                $min_date = $min_max_date->min_date;
                $max_date = $min_max_date->max_date;


                $request->merge(['current_year' => $fin_year]);

                $profit_and_loss_amt = $this->get_profit_and_loss_amount($request); 

                // If not in the last iteration
                if ($i < count($update_fin_years) - 1) { 
                        $opening_for_next_year = Balances::where('account_id', 1)
                            ->where('fin_year', $update_fin_years[$i + 1])
                            ->first();

                        $closing_for_current_year = Balances::where('account_id', 1)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        if(!$opening_for_next_year){
                            $opening_for_next_year = new Balances();
                            $opening_for_next_year->fin_year = $update_fin_years[$i + 1];
                            $opening_for_next_year->account_id = 1;
                        }
                        
                        if ($opening_for_next_year) {
                            if ($profit_and_loss_amt['profit_and_loss'] < 0) {
                                $opening_for_next_year->opening_bal_dr = 0.00;
                                $opening_for_next_year->opening_bal_cr = 0.00;

                                if($closing_for_current_year){
                                    $closing_for_current_year->closing_bal_dr = abs($profit_and_loss_amt['profit_and_loss']);
                                    $closing_for_current_year->closing_bal_cr = 0.00;
                                    $closing_for_current_year->closing_bal_date = $max_date;
                                    $closing_for_current_year->save();
                                }
                            } else {
                                $opening_for_next_year->opening_bal_dr = 0.00;
                                $opening_for_next_year->opening_bal_cr = 0.00;

                                if($closing_for_current_year){
                                    $closing_for_current_year->closing_bal_dr = 0.00;
                                    $closing_for_current_year->closing_bal_cr = abs($profit_and_loss_amt['profit_and_loss']);
                                    $closing_for_current_year->closing_bal_date = $max_date;
                                    $closing_for_current_year->save();
                                }
                            }
        
                            $min_date_for_next = $this->get_min_max_date($update_fin_years[$i + 1])->min_date;
                            $opening_for_next_year->opening_bal_date = $min_date_for_next;
        
                            $opening_for_next_year->save();
                        }
                    }
                    else{
                        $closing_for_last_year = Balances::where('account_id', 1)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        
                        if ($closing_for_last_year) {
                            if ($profit_and_loss_amt['profit_and_loss'] < 0) {
                                $closing_for_last_year->closing_bal_dr = abs($profit_and_loss_amt['profit_and_loss']);
                                $closing_for_last_year->closing_bal_cr = 0.00;
                            } else {
                                $closing_for_last_year->closing_bal_dr = 0.00;
                                $closing_for_last_year->closing_bal_cr = abs($profit_and_loss_amt['profit_and_loss']);
                            }
        
                            $max_date_for_current = $this->get_min_max_date($update_fin_years[$i])->max_date;
                            $closing_for_last_year->closing_bal_date = $max_date_for_current;
        
                            $closing_for_last_year->save();
                        }
                    }




             }

             // Loop through the array and update the balances OF CAPITAL
             for ($i = 0; $i < count($update_fin_years); $i++) {
                $fin_year = $update_fin_years[$i];
    
                $min_max_date = $this->get_min_max_date($fin_year);
                $min_date = $min_max_date->min_date;
                $max_date = $min_max_date->max_date;


                $request->merge(['current_year' => $fin_year]);

                $profit_and_loss_amt = $this->get_profit_and_loss_amount($request); 

                $capital_account_id = $this->get_acc_id('Capital Account');

                if(!$capital_account_id){
                    return response('Capital Account Not Found',500);
                }


                $sum_of_pl += $profit_and_loss_amt['profit_and_loss'];

                // If not in the last iteration
                if ($i < count($update_fin_years) - 1) { 
                        $capital_opening_for_next_year = Balances::where('account_id', $capital_account_id)
                            ->where('fin_year', $update_fin_years[$i + 1])
                            ->first();

                        $capital_closing_for_current_year = Balances::where('account_id', $capital_account_id)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        if(!$capital_opening_for_next_year){
                            $capital_opening_for_next_year = new Balances();
                            $capital_opening_for_next_year->fin_year = $update_fin_years[$i + 1];
                            $capital_opening_for_next_year->account_id = $capital_account_id;
                        }
                        
                        if ($capital_opening_for_next_year) {

                                if($capital_closing_for_current_year){
                                   if($capital_closing_for_current_year->closing_bal_dr>0){
                                        $closing_factor = -($capital_closing_for_current_year->closing_bal_dr) + $sum_of_pl;
                                   }
                                   else{
                                       $closing_factor = $capital_closing_for_current_year->closing_bal_cr + $sum_of_pl;
                                   }

                                   if($closing_factor>=0){
                                    $capital_closing_for_current_year->closing_bal_cr = abs($closing_factor);
                                    $capital_closing_for_current_year->closing_bal_dr = 0.00;
                                   }
                                   else{
                                    $capital_closing_for_current_year->closing_bal_dr = abs($closing_factor);
                                    $capital_closing_for_current_year->closing_bal_cr = 0.00;
                                   }
                                }

                                $capital_opening_for_next_year->opening_bal_dr = $capital_closing_for_current_year->closing_bal_dr;
                                $capital_opening_for_next_year->opening_bal_cr = $capital_closing_for_current_year->closing_bal_cr;
        
                            $min_date_for_next = $this->get_min_max_date($update_fin_years[$i + 1])->min_date;
                            $capital_opening_for_next_year->opening_bal_date = $min_date_for_next;
        
                            $capital_closing_for_current_year->save();
                            $capital_opening_for_next_year->save();
                        }
                    }
                    else{
                        $closing_for_last_year = Balances::where('account_id', $capital_account_id)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();

                        if($closing_for_last_year){
                            if($closing_for_last_year->closing_bal_dr>0){
                                 $closing_factor = -($closing_for_last_year->closing_bal_dr) + $sum_of_pl;
                            }
                            else{
                                $closing_factor = $closing_for_last_year->closing_bal_cr + $sum_of_pl;
                            }

                            if($closing_factor>=0){
                             $closing_for_last_year->closing_bal_cr = abs($closing_factor);
                             $closing_for_last_year->closing_bal_dr = 0.00;
                            }
                            else{
                             $closing_for_last_year->closing_bal_dr = abs($closing_factor);
                             $closing_for_last_year->closing_bal_cr = 0.00;
                            }

                            $max_date_for_current = $this->get_min_max_date($update_fin_years[$i])->max_date;
                            $closing_for_last_year->closing_bal_date = $max_date_for_current;
        
                            $closing_for_last_year->save();
                        }
                        
                        
                    }


            }
        }

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function get_analytics_report(Request $request){
        $branch = $request->report_data['branch'];
        $vehicle = $request->report_data['vehicle'];
        $acc_party = $request->report_data['acc_party'];
        $selection_type = $request->report_data['selection_type'];
        $type_of_report = $request->report_data['type_of_report'];
        $from_date = $request->report_data['from_date'];
        $to_date = $request->report_data['to_date'];

        $month = $request->report_data['month'];

        $currentYear = $request->current_year;

        $return_data = [];
        $truck_data = [];

        if($selection_type=='Monthly Vehicle P & L' || $selection_type=='Vehicle P & L'){

            if($vehicle){
                $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->where('id',$vehicle['id'])->get();
            }
            else{
                $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($trucks as $truck) {
                $truck_id = $truck->id;
                $truck_no = $truck->vehicle_number;

                $data = [];

                if( $selection_type == 'Monthly Vehicle P & L'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $truck_data = $this->get_truck_data($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, null);
                            
                            $truck_data['account_id'] = $truck['account_id'];

                            $truck_data['month'] = $i;

                            $truck_data['value'] = $truck_data['cr_amount'] - $truck_data['dr_amount'];

                            $truck_data['label'] = $this->getMonthName($i);

                            $truck_data['name'] = $truck_no;
                
                            if($truck_data['dr_amount']>0 || $truck_data['cr_amount']>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Vehicle P & L'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date =  $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date =  $to_date ? $to_date : $min_max_dates->max_date;

                    $truck_data = $this->get_truck_data($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, null);
                            
                    $truck_data['account_id'] = $truck['account_id'];

                    $truck_data['value'] = $truck_data['cr_amount'] - $truck_data['dr_amount'];

                    $truck_data['label'] = $truck_no;
                    $truck_data['name'] = $truck_no;
                
                    if($truck_data['dr_amount']>0 || $truck_data['cr_amount']>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Monthly Vehicle Diesel Exp' || $selection_type=='Vehicle Diesel Exp'){

            if($vehicle){
                $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->where('id',$vehicle['id'])->get();
            }
            else{
                $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->where('self_market','SELF')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($trucks as $truck) {
                $truck_id = $truck->id;
                $truck_no = $truck->vehicle_number;

                $data = [];

                if( $selection_type == 'Monthly Vehicle Diesel Exp'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = DieselReceipt::where('vehicle_id', $truck_id)->whereBetween('booking_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('amount');
                            
                            $truck_data['vehicle_id'] = $truck_id;
                            $truck_data['name'] = $truck_no;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Vehicle Diesel Exp'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = DieselReceipt::where('vehicle_id', $truck_id)->whereBetween('booking_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('amount');
                    
                    $truck_data['vehicle_id'] = $truck_id;
                    $truck_data['name'] = $truck_no;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $truck_no;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Branch Wise GR Amt' || $selection_type=='Monthly Branch Wise GR Amt'){

            if($branch){
                $branches = Branch::select('id', 'branch_name', 'branch_code')->where('id',$branch['id'])->get();
            }
            else{
                $branches = Branch::select('id', 'branch_name', 'branch_code')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($branches as $branch) {
                $branch_id = $branch->id;
                $branch_name = $branch->branch_name;

                $data = [];

                if( $selection_type == 'Monthly Branch Wise GR Amt'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = BookGr::where('branch_id', $branch_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                            
                            $truck_data['branch_id'] = $branch_id;
                            $truck_data['name'] = $branch_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Branch Wise GR Amt'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = BookGr::where('branch_id', $branch_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                    
                    $truck_data['branch_id'] = $branch_id;
                    $truck_data['name'] = $branch_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $branch_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Branch Wise LS Amt' || $selection_type=='Monthly Branch Wise LS Amt'){

            if($branch){
                $branches = Branch::select('id', 'branch_name', 'branch_code')->where('id',$branch['id'])->get();
            }
            else{
                $branches = Branch::select('id', 'branch_name', 'branch_code')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($branches as $branch) {
                $branch_id = $branch->id;
                $branch_name = $branch->branch_name;

                $data = [];

                if( $selection_type == 'Monthly Branch Wise LS Amt'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = LoadingSlip::where('branch_id', $branch_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                            
                            $truck_data['branch_id'] = $branch_id;
                            $truck_data['name'] = $branch_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Branch Wise LS Amt'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = LoadingSlip::where('branch_id', $branch_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                    
                    $truck_data['branch_id'] = $branch_id;
                    $truck_data['name'] = $branch_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $branch_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Branch Wise GR Count' || $selection_type=='Monthly Branch Wise GR Count'){

            if($branch){
                $branches = Branch::select('id', 'branch_name', 'branch_code')->where('id',$branch['id'])->get();
            }
            else{
                $branches = Branch::select('id', 'branch_name', 'branch_code')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($branches as $branch) {
                $branch_id = $branch->id;
                $branch_name = $branch->branch_name;

                $data = [];

                if( $selection_type == 'Monthly Branch Wise GR Count'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = BookGr::where('branch_id', $branch_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                            
                            $truck_data['branch_id'] = $branch_id;
                            $truck_data['name'] = $branch_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Branch Wise GR Count'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = BookGr::where('branch_id', $branch_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                    
                    $truck_data['branch_id'] = $branch_id;
                    $truck_data['name'] = $branch_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $branch_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Branch Wise LS Count' || $selection_type=='Monthly Branch Wise LS Count'){

            if($branch){
                $branches = Branch::select('id', 'branch_name', 'branch_code')->where('id',$branch['id'])->get();
            }
            else{
                $branches = Branch::select('id', 'branch_name', 'branch_code')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($branches as $branch) {
                $branch_id = $branch->id;
                $branch_name = $branch->branch_name;

                $data = [];

                if( $selection_type == 'Monthly Branch Wise LS Count'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = LoadingSlip::where('branch_id', $branch_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                            
                            $truck_data['branch_id'] = $branch_id;
                            $truck_data['name'] = $branch_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Branch Wise LS Count'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = LoadingSlip::where('branch_id', $branch_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                    
                    $truck_data['branch_id'] = $branch_id;
                    $truck_data['name'] = $branch_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $branch_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Party Wise GR Amt' || $selection_type=='Monthly Party Wise GR Amt'){

            if($acc_party){
                $parties = Party::select('id', 'party_name', 'account_id')->where('account_id',$acc_party['id'])->get();
            }
            else{
                $parties = Party::select('id', 'party_name', 'account_id')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($parties as $party) {
                $party_id = $party->account_id;
                $party_name = $party->party_name;

                $data = [];

                if( $selection_type == 'Monthly Party Wise GR Amt'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = BookGr::where('acc_party_id', $party_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                            
                            $truck_data['party_id'] = $party_id;
                            $truck_data['name'] = $party_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Party Wise GR Amt'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = BookGr::where('acc_party_id', $party_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                    
                    $truck_data['party_id'] = $party_id;
                    $truck_data['name'] = $party_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $party_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Party Wise LS Amt' || $selection_type=='Monthly Party Wise LS Amt'){

            if($acc_party){
                $parties = Party::select('id', 'party_name', 'account_id')->where('account_id',$acc_party['id'])->get();
            }
            else{
                $parties = Party::select('id', 'party_name', 'account_id')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($parties as $party) {
                $party_id = $party->account_id;
                $party_name = $party->party_name;

                $data = [];

                if( $selection_type == 'Monthly Party Wise LS Amt'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = LoadingSlip::where('acc_party_id', $party_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                            
                            $truck_data['party_id'] = $party_id;
                            $truck_data['name'] = $party_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Party Wise LS Amt'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = LoadingSlip::where('acc_party_id', $party_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->sum('grand_total');
                    
                    $truck_data['party_id'] = $party_id;
                    $truck_data['name'] = $party_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $party_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Party Wise GR Count' || $selection_type=='Monthly Party Wise GR Count'){

            if($acc_party){
                $parties = Party::select('id', 'party_name', 'account_id')->where('account_id',$acc_party['id'])->get();
            }
            else{
                $parties = Party::select('id', 'party_name', 'account_id')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($parties as $party) {
                $party_id = $party->account_id;
                $party_name = $party->party_name;

                $data = [];

                if( $selection_type == 'Monthly Party Wise GR Count'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = BookGr::where('acc_party_id', $party_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                            
                            $truck_data['party_id'] = $party_id;
                            $truck_data['name'] = $party_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Party Wise GR Count'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = BookGr::where('acc_party_id', $party_id)->whereBetween('gr_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                    
                    $truck_data['party_id'] = $party_id;
                    $truck_data['name'] = $party_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $party_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        if($selection_type=='Party Wise LS Count' || $selection_type=='Monthly Party Wise LS Count'){

            if($acc_party){
                $parties = Party::select('id', 'party_name', 'account_id')->where('account_id',$acc_party['id'])->get();
            }
            else{
                $parties = Party::select('id', 'party_name', 'account_id')->get();
            }
    
            
    
            $data = [];
    

            list($startYear, $endYear) = explode('-', $currentYear);

            foreach ($parties as $party) {
                $party_id = $party->account_id;
                $party_name = $party->party_name;

                $data = [];

                if( $selection_type == 'Monthly Party Wise LS Count'){
                    for ($i=1; $i <= 12; $i++) { 

                        if($month && $month!=$i){
                            continue;
                        }    

                            if ($i >= 1 && $i <= 3) {
                                $actualYear = $endYear; // Jan, Feb, Mar belong to the next year
                            } else {
                                $actualYear = $startYear; // Apr to Dec belong to the start year
                            }

                            // Get the first and last day of the month
                            $min_date = date('Y-m-d', strtotime('first day of '.$actualYear.'-'.$i));
                            $max_date = date('Y-m-d', strtotime('last day of '.$actualYear.'-'.$i));

                            $from_date = $min_date;
                            $to_date = $max_date;
        
                            $gr_query_sum = LoadingSlip::where('acc_party_id', $party_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                            
                            $truck_data['party_id'] = $party_id;
                            $truck_data['name'] = $party_name;
                            $truck_data['month'] = $i;

                            $truck_data['value'] = $gr_query_sum;

                            $truck_data['label'] = $this->getMonthName($i);
                
                            if($gr_query_sum>0){
                                $data[] = $truck_data;
                            }
                        

                    }
                }

                if( $selection_type == 'Party Wise LS Count'){
                    $min_max_dates = $this->get_min_max_date($currentYear);

                    $min_date = $min_max_dates->min_date;
                    $from_date = $from_date ? $from_date : $min_max_dates->min_date;
                    $to_date = $to_date ? $to_date : $min_max_dates->max_date;

                    $gr_query_sum = LoadingSlip::where('acc_party_id', $party_id)->whereBetween('ls_date', [$from_date, $to_date])->where('fin_year',$currentYear)->count();
                    
                    $truck_data['party_id'] = $party_id;
                    $truck_data['name'] = $party_name;
                    $truck_data['value'] = $gr_query_sum;

                    $truck_data['label'] = $party_name;
                
                    if($gr_query_sum>0){
                        $data = $truck_data;
                    }
                }

                $return_data[] = $data;
    
            }
            
        }

        return response()->json(['data'=>$return_data]);
        

    }


    protected function get_profit_and_loss_amount($request){
        $data = $this->get_profit_and_loss($request)->original;


        $expense_side = $data['expense_side'];
        $income_side = $data['income_side'];

        $expense_total = 0;
        $income_total = 0;

        foreach ($expense_side as $expense) {
            $expense_total += $expense['group_total_dr'] - $expense['group_total_cr'];
        }

        foreach ($income_side as $income) {
            $income_total += $income['group_total_cr'] - $income['group_total_dr'];
        }

        $profit_and_loss = $income_total - $expense_total;



        return ['expense_total' => $expense_total, 'income_total' => $income_total, 'profit_and_loss' => $profit_and_loss];
    }

    protected function get_truck_data($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, $branch_id) {
        $truck_data = [
            'vehicle_id' => $truck_id,
            'vehicle_number' => $truck_no,
            'dr_amount' => 0,
            'cr_amount' => 0,
        ];

        $payment_voucher_sum = VoucherDesc::where('against', 'VEHICLE_COST_CENTER')
                            ->where('against_id', $truck_id)
                            ->whereIn('voucher_name', ['Payment Voucher', 'Journal'])
                            ->where('fin_year', $currentYear)
                            ->whereHas('voucher', function ($query) use ($from_date, $to_date,$branch_id) {
                                $query->where('voucher_date', '>=', $from_date)
                                    ->where('voucher_date', '<=', $to_date)
                                    ->when($branch_id, function ($query, $branch_id) {
                                        return $query->where('branch_id', $branch_id);
                                    });
                            })
                            ->sum('receipt_amt');

        $driver_settlement_sum = DriverSettlement::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('settle_date', '>=', $from_date)
                            ->where('settle_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->where('driver_expense', '>=', 0)
                            ->sum('driver_expense');
        
        $driver_settlement_sum2 = DriverSettlement::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('settle_date', '>=', $from_date)
                            ->where('settle_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->where('driver_expense', '<', 0)
                            ->sum('driver_expense');

       
        $diesel_receipt_sum = DieselReceipt::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('booking_date', '>=', $from_date)
                            ->where('booking_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('amount');

        $sum_of_qty_in_purchase = PurchaseDesc::where('vehicle_id',$truck_id)
                                    ->whereHas('purchase', function ($query) use ($from_date, $to_date,$branch_id) {
                                        $query->where('bill_date', '>=', $from_date)
                                            ->where('bill_date', '<=', $to_date)
                                            ->where('bill_name', '!=', 'Sale')
                                            ->when($branch_id, function ($query, $branch_id) {
                                                return $query->where('branch_id', $branch_id);
                                            });
                                    })
                                   ->where('fin_year',$currentYear)
                                   ->sum('net_amount');

        $sum_of_amt_in_bill_desc = BillDesc::where('vehicle_id',$truck_id)
                                   ->whereHas('bill', function ($query) use ($from_date, $to_date,$branch_id) {
                                       $query->where('bill_date', '>=', $from_date)
                                           ->where('bill_date', '<=', $to_date)
                                           ->when($branch_id, function ($query, $branch_id) {
                                               return $query->where('branch_id', $branch_id);
                                           });
                                   })
                                   ->with(['bill:id,bill_number,bill_date,remarks,income','bill.income:id,account_name','vehicle:id,vehicle_number'])
                                  ->where('fin_year',$currentYear)
                                  ->sum('grand_total');

                        
        $pods_sum = Pod::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('delivery_date', '>=', $from_date)
                            ->where('delivery_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('grand_total');
                        
        $gr_without_pod_sum = BookGr::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->whereDoesntHave('pod_desc')
                            ->sum('grand_total');
        
        $lorry_hire_expense = BookGr::select('id','gr_number','gr_date','dr_on_account','lh_net_freight')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->where('lh_net_freight','>',0)
                            ->with(['dr_on_account:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('lh_net_freight');
                           
        $challan = Challan::select('id','challan_number','challan_date','vehicle_id','dr_on_account','lh_net_freight','remarks')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('challan_date', '>=', $from_date)
                            ->where('challan_date', '<=', $to_date)
                            ->with(['dr_on_account:id,account_name'])
                            ->where('lh_net_freight','>',0)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('lh_net_freight');
                        
        $loading_slip_sum = LoadingSlip::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('ls_date', '>=', $from_date)
                            ->where('ls_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('total_without_gst');
        
                    $driver_advance_sum = BookGr::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('driver_payment_amt');

        $debit_minus_credit = ($payment_voucher_sum + $diesel_receipt_sum + $sum_of_qty_in_purchase + $driver_advance_sum + $driver_settlement_sum + $lorry_hire_expense + $challan) - ($pods_sum + $gr_without_pod_sum + $sum_of_amt_in_bill_desc + $loading_slip_sum + abs($driver_settlement_sum2));

        $truck_data['dr_amount'] = $debit_minus_credit > 0 ? $debit_minus_credit : 0;
        $truck_data['cr_amount'] = $debit_minus_credit < 0 ? abs($debit_minus_credit) : 0;

        return $truck_data;
    }

    protected function get_truck_data_with_sums($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, $branch_id) {
        $truck_data = [
            'vehicle_id' => $truck_id,
            'vehicle_number' => $truck_no,
            'total_gr' => 0,
            'total_ls' => 0,
            'total_pod' => 0,
            'total_purchase' => 0,
            'total_payment' => 0,
            'total_diesel' => 0,
            'total_lh' => 0,
            'total_trip' => 0,

            'total_gr_amount' => 0,
            'total_pod_amount' => 0,
            'total_payment_amount' => 0,
            'total_ls_amount' => 0,
            'total_lh_amount' => 0,
            'total_diesel_amount' => 0,
            'total_trip_amount' => 0,

            'dr_amount' => 0,
            'cr_amount' => 0,
        ];

        $payment_voucher = VoucherDesc::where('against', 'VEHICLE_COST_CENTER')
                            ->where('against_id', $truck_id)
                            ->whereIn('voucher_name', ['Payment Voucher', 'Journal'])
                            ->where('fin_year', $currentYear)
                            ->whereHas('voucher', function ($query) use ($from_date, $to_date,$branch_id) {
                                $query->where('voucher_date', '>=', $from_date)
                                    ->where('voucher_date', '<=', $to_date)
                                    ->when($branch_id, function ($query, $branch_id) {
                                        return $query->where('branch_id', $branch_id);
                                    });
                            })
                            ->selectRaw('COUNT(*) as count, SUM(receipt_amt) as sum')
                            ->first();

        $payment_voucher_sum =  $payment_voucher->sum;
        $payment_voucher_count = $payment_voucher->count;

       
        $diesel_receipt_info = DieselReceipt::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('booking_date', '>=', $from_date)
                            ->where('booking_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->selectRaw('COUNT(*) as count, SUM(amount) as sum')
                            ->first();
                        
                        $diesel_receipt_count = $diesel_receipt_info->count;
                        $diesel_receipt_sum = $diesel_receipt_info->sum;
                        
        $purchase_info = PurchaseDesc::where('vehicle_id', $truck_id)
                        ->whereHas('purchase', function ($query) use ($from_date, $to_date, $branch_id) {
                            $query->where('bill_date', '>=', $from_date)
                                ->where('bill_date', '<=', $to_date)
                                ->where('bill_name', '!=', 'Sale')
                                ->when($branch_id, function ($query, $branch_id) {
                                    return $query->where('branch_id', $branch_id);
                                });
                        })
                        ->where('fin_year', $currentYear)
                        ->selectRaw('COUNT(*) as count, SUM(net_amount) as sum')
                        ->first();
                    
                    $sum_of_qty_in_purchase = $purchase_info->sum;
                    $purchase_count = $purchase_info->count;
                 
        $bill_desc_info = BillDesc::where('vehicle_id',$truck_id)
                        ->whereHas('bill', function ($query) use ($from_date, $to_date,$branch_id) {
                            $query->where('bill_date', '>=', $from_date)
                                ->where('bill_date', '<=', $to_date)
                                ->when($branch_id, function ($query, $branch_id) {
                                    return $query->where('branch_id', $branch_id);
                                });
                        })
                        ->with(['bill:id,bill_number,bill_date,remarks,income','bill.income:id,account_name','vehicle:id,vehicle_number'])
                        ->selectRaw('COUNT(*) as count, SUM(grand_total) as sum')
                        ->first();

                        $sum_of_amt_in_bill_desc = $bill_desc_info->sum;
                    $bill_desc_count = $bill_desc_info->count;
                        
        $pods_info = Pod::where('vehicle_id', $truck_id)
                    ->where('fin_year', $currentYear)
                    ->where('delivery_date', '>=', $from_date)
                    ->where('delivery_date', '<=', $to_date)
                    ->when($branch_id, function ($query, $branch_id) {
                        return $query->where('branch_id', $branch_id);
                    })
                    ->selectRaw('COUNT(*) as count, SUM(grand_total) as sum')
                    ->first();
                
                $pods_sum = $pods_info->sum;
                $pods_count = $pods_info->count;
                
        $gr_without_pod_info = BookGr::where('vehicle_id', $truck_id)
                    ->where('fin_year', $currentYear)
                    ->where('gr_date', '>=', $from_date)
                    ->where('gr_date', '<=', $to_date)
                    ->when($branch_id, function ($query, $branch_id) {
                        return $query->where('branch_id', $branch_id);
                    })
                    ->whereDoesntHave('pod_desc')
                    ->selectRaw('COUNT(*) as count, SUM(grand_total) as sum')
                    ->first();
                
        $gr_without_pod_sum = $gr_without_pod_info->sum;
        $gr_without_pod_count = $gr_without_pod_info->count;

        $lorry_hire_info = BookGr::where('vehicle_id', $truck_id)
                    ->where('fin_year', $currentYear)
                    ->where('gr_date', '>=', $from_date)
                    ->where('gr_date', '<=', $to_date)
                    ->when($branch_id, function ($query, $branch_id) {
                        return $query->where('branch_id', $branch_id);
                    })
                    ->selectRaw('COUNT(*) as count, SUM(lh_net_freight) as sum')
                    ->first();
                
        $lorry_hire_sum = $lorry_hire_info->sum;
        $lorry_hire_count = $lorry_hire_info->count;

        $challan = Challan::where('vehicle_id', $truck_id)
                  ->where('fin_year', $currentYear)
                  ->where('challan_date', '>=', $from_date)
                  ->where('challan_date', '<=', $to_date)
                  ->where('lh_net_freight', '>', 0)
                  ->when($branch_id, function ($query, $branch_id) {
                      return $query->where('branch_id', $branch_id);
                  })
                  ->selectRaw('COUNT(id) as count, SUM(lh_net_freight) as sum')
                  ->first();

        $challan_sum = $challan->sum;
        $challan_count = $challan->count;


        
                
        $loading_slip_info = LoadingSlip::where('vehicle_id', $truck_id)
                    ->where('fin_year', $currentYear)
                    ->where('ls_date', '>=', $from_date)
                    ->where('ls_date', '<=', $to_date)
                    ->when($branch_id, function ($query, $branch_id) {
                        return $query->where('branch_id', $branch_id);
                    })
                    ->selectRaw('COUNT(*) as count, SUM(total_without_gst) as sum')
                    ->first();
                
                $loading_slip_sum = $loading_slip_info->sum;
                $loading_slip_count = $loading_slip_info->count;
        
        $driver_settlement_sum = DriverSettlement::where('vehicle_id', $truck_id)
                ->where('fin_year', $currentYear)
                ->where('settle_date', '>=', $from_date)
                ->where('settle_date', '<=', $to_date)
                ->when($branch_id, function ($query, $branch_id) {
                    return $query->where('branch_id', $branch_id);
                })
                ->where('driver_expense', '>=', 0)
                ->selectRaw('COUNT(*) as count, SUM(driver_expense) as sum')
                ->first();

        

       $driver_settlement_sum2 = DriverSettlement::where('vehicle_id', $truck_id)
                ->where('fin_year', $currentYear)
                ->where('settle_date', '>=', $from_date)
                ->where('settle_date', '<=', $to_date)
                ->when($branch_id, function ($query, $branch_id) {
                    return $query->where('branch_id', $branch_id);
                })
                ->where('driver_expense', '<', 0)
                ->selectRaw('COUNT(*) as count, SUM(driver_expense) as sum')
                ->first();

                $driver_advance = BookGr::where('vehicle_id', $truck_id)
                ->where('fin_year', $currentYear)
                ->where('gr_date', '>=', $from_date)
                ->where('gr_date', '<=', $to_date)
                ->when($branch_id, function ($query, $branch_id) {
                    return $query->where('branch_id', $branch_id);
                })
                ->where('driver_payment_amt', '>', 0)
                ->selectRaw('COUNT(*) as count, SUM(driver_payment_amt) as sum')
                ->first();
        $driver_advance_sum = $driver_advance->sum;
        $driver_advance_count = $driver_advance->count;

        $debit_minus_credit = ($payment_voucher_sum + $diesel_receipt_sum + $sum_of_qty_in_purchase  + $driver_advance_sum + $driver_settlement_sum->sum + $lorry_hire_sum + $challan_sum) - ($pods_sum + $gr_without_pod_sum + $sum_of_amt_in_bill_desc + $loading_slip_sum + abs($driver_settlement_sum2->sum));

        
        $truck_data['dr_amount'] = $debit_minus_credit > 0 ? $debit_minus_credit : 0;
        $truck_data['cr_amount'] = $debit_minus_credit < 0 ? abs($debit_minus_credit) : 0;

        $truck_data['total_gr'] = $gr_without_pod_count;
        $truck_data['total_ls'] = $loading_slip_count;
        $truck_data['total_pod'] = $pods_count;
        $truck_data['total_payment'] = $payment_voucher_count;
        $truck_data['total_purchase'] = $purchase_count;
        $truck_data['total_diesel'] = $diesel_receipt_count;
        $truck_data['total_lh'] = $lorry_hire_count + $challan_count;
        $truck_data['total_trip'] = 0;

        $truck_data['total_gr_amount'] = $gr_without_pod_sum;
        $truck_data['total_pod_amount'] = $pods_sum;
        $truck_data['total_payment_amount'] = $payment_voucher_sum;
        $truck_data['total_ls_amount'] = $loading_slip_sum;
        $truck_data['total_lh_amount'] = $lorry_hire_sum +$challan_sum ;
        $truck_data['total_diesel_amount'] = $diesel_receipt_sum;
        $truck_data['total_purchase_amount'] = $sum_of_qty_in_purchase;
        $truck_data['total_trip_amount'] = $driver_settlement_sum->sum + $driver_settlement_sum2->sum + $driver_advance_sum;

        return $truck_data;
    }

    function getMonthName($monthNumber) {
        $months = [
            1 => 'January',
            2 => 'February',
            3 => 'March',
            4 => 'April',
            5 => 'May',
            6 => 'June',
            7 => 'July',
            8 => 'August',
            9 => 'September',
            10 => 'October',
            11 => 'November',
            12 => 'December'
        ];
    
        return isset($months[$monthNumber]) ? $months[$monthNumber] : null;
    }

    

    
    protected function get_data_for_obj($pk,$currentYear)
    {
        // Fetch and send the account
        $account = Account::select('id','account_name','account_type','branch_id','group_id')->with(['group:id,group_name,is_primary','balances'=> function ($query) use ($currentYear) {
            $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear)->first();
        }])->find($pk);

        return ['account' => $account];
    }


    
}
