<?php

namespace App\Http\Controllers\Bill;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Bill\Bill;
use App\Models\Bill\BillDesc;
use App\Models\Bill\CreditNote;
use App\Models\Bill\CreditNoteDesc;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\Inventory\Purchase;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class CreditNoteController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'CreditNote');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $credit_notes = CreditNote::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'cn_desc',
            'income:id,account_name'
        ])->where('fin_year',$request->current_year)->get();

        $credit_notes->cn_desc->transform(function ($desc) { 
            return [
                'cn_desc_id' => $desc->id,
                'cn_id' => $desc->cn_id,
                'against' => $desc->against,
                'bill_id' => $desc->bill_id,
                'bill' => ['id'=>$desc->bill_id, 'bill_number'=>$desc->bill_number],
                'bill_date' => $desc->bill_date,
                'gr_number' => $desc->gr_number,
                'amount' => $desc->amount,
                'net_amount' => $desc->net_amount,
                'igst_percent' => $desc->igst_percent,
                'cgst_percent' => $desc->cgst_percent,
                'sgst_percent' => $desc->sgst_percent,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
            ];
        });

        return response()->json($credit_notes);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $credit_note = CreditNote::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'cn_desc',
            'income:id,account_name'
        ])->find($pk);

        $credit_note->cn_desc->transform(function ($desc) {     
                return [
                    'cn_desc_id' => $desc->id,
                    'cn_id' => $desc->cn_id,
                    'against' => $desc->against,
                    'bill_id' => $desc->bill_id,
                    'bill' => ['id'=>$desc->bill_id, 'bill_number'=>$desc->bill_number],
                    'bill_date' => $desc->bill_date,
                    'gr_number' => $desc->gr_number,
                    'amount' => $desc->amount,
                    'net_amount' => $desc->net_amount,
                    'igst_percent' => $desc->igst_percent,
                    'cgst_percent' => $desc->cgst_percent,
                    'sgst_percent' => $desc->sgst_percent,
                    'igst_amount' => $desc->igst_amount,
                    'cgst_amount' => $desc->cgst_amount,
                    'sgst_amount' => $desc->sgst_amount,
                ];
        });

        return response()->json(['credit_note' =>$credit_note]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'CreditNote');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'cn_number' => 'required|unique:credit_notes,cn_number',
            'acc_party' => 'required',
        ], [
            'branch.required' => 'Required',
            'cn_number.required' => 'Required',
            'cn_number.unique' => 'Already Exist !',
            'acc_party.required' => 'Required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE CHALLAN TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_cn_number($request->cn_number);
            
            // SAVING A NEW DS
            $credit_note = new CreditNote();
            $credit_note->branch_id = $request->branch['id'];
            $credit_note->max_number = $max_number;
            $credit_note->note_name = $request->note_name;
            $credit_note->cn_number = $request->cn_number;
            $credit_note->party_ref_number = $request->party_ref_number;
            $credit_note->cn_date = $request->cn_date ;
            $credit_note->acc_party_id = $request->acc_party['id'] ;
            $credit_note->income_id = $request->income['id'] ;
            $credit_note->remarks = $request->remarks ;

            $credit_note->v_total_without_tax = $request->v_total_without_tax ;
            $credit_note->v_igst_amount = $request->v_igst_amount ;
            $credit_note->v_cgst_amount = $request->v_cgst_amount ;
            $credit_note->v_sgst_amount = $request->v_sgst_amount ;
            $credit_note->v_grand_total =$request->v_grand_total;

            $credit_note->total_without_tax = $request->total_without_tax ;
            $credit_note->igst_amount = $request->igst_amount ;
            $credit_note->cgst_amount = $request->cgst_amount ;
            $credit_note->sgst_amount = $request->sgst_amount ;
            $credit_note->grand_total =$request->grand_total;

            $credit_note->fin_year = $request->current_year;

            $credit_note->created_by = $request->user->id;

            // Save the model to the database
            $credit_note->save();
            

            // SAVING SETTLEMENT DETAILS
            $cn_desc = $request->cn_desc;
            foreach ($cn_desc as $dsd) {
                $cn_desc = new CreditNoteDesc();
                $cn_desc->cn_id = $credit_note->id;
                $cn_desc->cn_number = $credit_note->cn_number;
                $cn_desc->against = $dsd['against'];
                $cn_desc->bill_id = $dsd['bill']['id'];
                $cn_desc->bill_number = $dsd['bill']['bill_number'];
                $cn_desc->bill_date = $dsd['bill_date'];
                $cn_desc->gr_number = $dsd['gr_number'];
                $cn_desc->amount = $dsd['amount'];
                $cn_desc->igst_percent = $dsd['igst_percent'];
                $cn_desc->cgst_percent = $dsd['cgst_percent'];
                $cn_desc->sgst_percent = $dsd['sgst_percent'];
                $cn_desc->igst_amount = $dsd['igst_amount'];
                $cn_desc->cgst_amount = $dsd['cgst_amount'];
                $cn_desc->sgst_amount = $dsd['sgst_amount'];
                $cn_desc->net_amount = $dsd['net_amount'];
                $cn_desc->fin_year = $request->current_year;
                $cn_desc->save();

                if($cn_desc->against = 'Bill'){
                    
                    $bill_data = Bill::select('id','bill_number','total_without_gst','other_plus','other_minus','selection_type')->find($cn_desc['bill_id']);

                    $sum_of_bill = $bill_data->total_without_gst + $bill_data->other_plus - $bill_data->other_minus;
                    if( $sum_of_bill ==  $cn_desc->amount ){
                        $bill_status = false;
                    }
                    else{
                        $bill_status = true;
                    }

                    // SPLIT gr_number based on ',' and update the status of only those GRs 
                    $gr_numbers = explode(',', $cn_desc->gr_number);
                    if($gr_numbers){
                        $bill_status = false;
                    }



                    $bill_desc_ids = BillDesc::select('id')->where('bill_id', $cn_desc->bill_id )
                                     ->when($gr_numbers, function ($query) use ($gr_numbers,$bill_data) {
                                        if($bill_data->selection_type == 'GR'){
                                            return $query->whereIn('gr_number', $gr_numbers);
                                        }
                                        else if($bill_data->selection_type == 'Loading Slip'){
                                            return $query->whereIn('ls_number', $gr_numbers);
                                        }
                                    })
                                    ->where('fin_year',$request->current_year)->pluck('id')->toArray();

                                   
                                    
                    foreach ($bill_desc_ids as $pdi) {
                        $bill_desc = BillDesc::select('gr_id','gr_number','ls_id','ls_number','selection_type','included_charges')->find($pdi);
                        if($bill_desc->selection_type == 'GR'){
                            $this->change_bill_status('GR', $bill_status, $bill_desc['gr_id'], $bill_desc->included_charges);
                        }
                        else if($bill_desc->selection_type == 'Loading Slip'){
                            $this->change_bill_status('Loading Slip',$bill_status,$bill_desc['ls_id'],'NONE');
                        }
                    }

                }

                
            }


            // RECORD THE TRANSACTION IN ACCOUNTS
           $this->record_transaction_in_accounts($request,$credit_note);
            // TRANSACTION RECORDED IN ACCOUNTS

             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($credit_note->id);
            $this->createEditLog('created', 'credit_notes', $credit_note->id, $credit_note->cn_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$credit_note->id, 'cn_number'=>$credit_note->cn_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'CreditNote');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $credit_note = CreditNote::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($credit_note->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'cn_number' => 'required',
            'acc_party' => 'required',
        ], [
            'branch.required' => 'Required',
            'cn_number.required' => 'Required',
            'acc_party.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW DS
            $credit_note->party_ref_number = $request->party_ref_number;
            $credit_note->cn_date = $request->cn_date ;
            $credit_note->acc_party_id = $request->acc_party['id'] ;
            $credit_note->income_id = $request->income['id'] ;
            $credit_note->remarks = $request->remarks ;

            $credit_note->v_total_without_tax = $request->v_total_without_tax ;
            $credit_note->v_igst_amount = $request->v_igst_amount ;
            $credit_note->v_cgst_amount = $request->v_cgst_amount ;
            $credit_note->v_sgst_amount = $request->v_sgst_amount ;
            $credit_note->v_grand_total =$request->v_grand_total;

            $credit_note->total_without_tax = $request->total_without_tax ;
            $credit_note->igst_amount = $request->igst_amount ;
            $credit_note->cgst_amount = $request->cgst_amount ;
            $credit_note->sgst_amount = $request->sgst_amount ;
            $credit_note->grand_total =$request->grand_total;


            $credit_note->updated_by = $request->user->id;

            $credit_note->save();
            // NEW CHALLAN SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_cn_desc = $request->newly_added_cn_desc;
            foreach ($newly_added_cn_desc as $nald) {
                $cn_desc = new CreditNoteDesc();
                $cn_desc->cn_id = $credit_note->id;
                $cn_desc->cn_number = $credit_note->cn_number;
                $cn_desc->against = $nald['against'];
                $cn_desc->bill_id = $nald['bill']['id'];
                $cn_desc->bill_number = $nald['bill']['bill_number'];
                $cn_desc->bill_date = $nald['bill_date'];
                $cn_desc->gr_number = $nald['gr_number'];
                $cn_desc->amount = $nald['amount'];
                $cn_desc->igst_percent = $nald['igst_percent'];
                $cn_desc->cgst_percent = $nald['cgst_percent'];
                $cn_desc->sgst_percent = $nald['sgst_percent'];
                $cn_desc->igst_amount = $nald['igst_amount'];
                $cn_desc->cgst_amount = $nald['cgst_amount'];
                $cn_desc->sgst_amount = $nald['sgst_amount'];
                $cn_desc->net_amount = $nald['net_amount'];
                $cn_desc->fin_year = $request->current_year;
                $cn_desc->save();

                if($cn_desc->against = 'Bill'){
                    $bill_data = Bill::select('id','bill_number','total_without_gst','other_plus','other_minus','selection_type')->find($cn_desc['bill_id']);

                    $sum_of_bill = $bill_data->total_without_gst + $bill_data->other_plus - $bill_data->other_minus;
                    if( $sum_of_bill ==  $cn_desc->amount ){
                        $bill_status = false;
                    }
                    else{
                        $bill_status = true;
                    }

                     // SPLIT gr_number based on ',' and update the status of only those GRs 
                     $gr_numbers = explode(',', $cn_desc->gr_number);
                     if($gr_numbers){
                         $bill_status = false;
                     }
 
 
 
                     $bill_desc_ids = BillDesc::select('id')->where('bill_id', $cn_desc->bill_id )
                                      ->when($gr_numbers, function ($query) use ($gr_numbers,$bill_data) {
                                         if($bill_data->selection_type == 'GR'){
                                             return $query->whereIn('gr_number', $gr_numbers);
                                         }
                                         else if($bill_data->selection_type == 'Loading Slip'){
                                             return $query->whereIn('ls_number', $gr_numbers);
                                         }
                                     })
                                     ->where('fin_year',$request->current_year)->pluck('id')->toArray();

                                     
                    $bill_desc_ids = BillDesc::select('id')->where('bill_id', $cn_desc->bill_id )->where('fin_year',$request->current_year)->pluck('id')->toArray();
                    foreach ($bill_desc_ids as $pdi) {
                        $bill_desc = BillDesc::select('gr_id','gr_number','ls_id','ls_number','selection_type','included_charges')->find($pdi);
                        if($bill_desc->selection_type == 'GR'){
                            $this->change_bill_status('GR', $bill_status, $bill_desc['gr_id'], $bill_desc->included_charges);
                        }
                        else if($bill_desc->selection_type == 'Loading Slip'){
                            $this->change_bill_status('Loading Slip',$bill_status,$bill_desc['ls_id'],'NONE');
                        }
                    }
                }
            }
            // PACKAGE DETAILS SAVED

            // UPDATED
            $updated_cn_desc = $request->updated_cn_desc;
            foreach($updated_cn_desc as $upcnd){
                $up_cn = CreditNoteDesc::find($upcnd['cn_desc_id']);
                $up_cn->amount = $upcnd['amount'];
                $up_cn->igst_percent = $upcnd['igst_percent'];
                $up_cn->cgst_percent = $upcnd['cgst_percent'];
                $up_cn->sgst_percent = $upcnd['sgst_percent'];
                $up_cn->igst_amount = $upcnd['igst_amount'];
                $up_cn->cgst_amount = $upcnd['cgst_amount'];
                $up_cn->sgst_amount = $upcnd['sgst_amount'];
                $up_cn->net_amount = $upcnd['net_amount'];
                $up_cn->save();


                if($up_cn->against = 'Bill'){

                    $bill_data = Bill::select('id','bill_number','total_without_gst','other_plus','other_minus','selection_type')->find($upcnd['bill_id']);

                    $sum_of_bill = $bill_data->total_without_gst + $bill_data->other_plus - $bill_data->other_minus;

                    if( $sum_of_bill ==  $up_cn->amount ){
                        $bill_status = false;
                    }
                    else{
                        $bill_status = true;
                    }

                     // SPLIT gr_number based on ',' and update the status of only those GRs 
                     $gr_numbers = explode(',', $up_cn->gr_number);
                     if($gr_numbers){
                         $bill_status = false;
                     }
 
 
 
                     $bill_desc_ids = BillDesc::select('id')->where('bill_id', $up_cn->bill_id )
                                      ->when($gr_numbers, function ($query) use ($gr_numbers,$bill_data) {
                                         if($bill_data->selection_type == 'GR'){
                                             return $query->whereIn('gr_number', $gr_numbers);
                                         }
                                         else if($bill_data->selection_type == 'Loading Slip'){
                                             return $query->whereIn('ls_number', $gr_numbers);
                                         }
                                     })
                                     ->where('fin_year',$request->current_year)->pluck('id')->toArray();



                    $bill_desc_ids = BillDesc::select('id')->where('bill_id', $upcnd['bill_id'] )->where('fin_year',$request->current_year)->pluck('id')->toArray();
                    foreach ($bill_desc_ids as $pdi) {
                        $bill_desc = BillDesc::select('gr_id','gr_number','ls_id','ls_number','selection_type','included_charges')->find($pdi);
                        if($bill_desc->selection_type == 'GR'){
                            $this->change_bill_status('GR', $bill_status, $bill_desc['gr_id'], $bill_desc->included_charges);
                        }
                        else if($bill_desc->selection_type == 'Loading Slip'){
                            $this->change_bill_status('Loading Slip',$bill_status,$bill_desc['ls_id'],'NONE');
                        }
                    }

                }

            }

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_cn_desc = $request->deleted_cn_desc;
            if( !empty($deleted_cn_desc) ){
                foreach ($deleted_cn_desc as $dcd) {
                    $lhd = CreditNoteDesc::find($dcd);
                    $lhd->delete();

                    if($lhd->against = 'Bill'){

                            $bill_desc_ids = BillDesc::select('id')->where('bill_id', $lhd->bill_id )->where('fin_year',$request->current_year)->pluck('id')->toArray();
                            foreach ($bill_desc_ids as $pdi) {
                                $bill_desc = BillDesc::select('gr_id','gr_number','ls_id','ls_number','selection_type','included_charges')->find($pdi);
                                if($bill_desc->selection_type == 'GR'){
                                    $this->change_bill_status('GR', true, $bill_desc['gr_id'], $bill_desc->included_charges);
                                }
                                else if($bill_desc->selection_type == 'Loading Slip'){
                                    $this->change_bill_status('Loading Slip',true,$bill_desc['ls_id'],'NONE');
                                }
                            }

                    }
                }
                    
            }
            //DELETED EXISTING PACKAGE DETAILS


            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request,$credit_note);
            // TRANSACTION RECORDED IN ACCOUNTS
            

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($credit_note->id);
            $this->createEditLog('updated', 'credit_notes', $credit_note->id, $credit_note->cn_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'CreditNote');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $credit_note = CreditNote::find($pk);


        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($credit_note->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

            
            $bill_ids = CreditNoteDesc::select('bill_id')->where('cn_id', $pk)->where('against','Bill')->where('fin_year',$request->current_year)->pluck('bill_id')->toArray();
            foreach($bill_ids as $b_id){
                $bill_desc_ids = BillDesc::select('id')->where('bill_id', $b_id )->where('fin_year',$request->current_year)->pluck('id')->toArray();
                foreach ($bill_desc_ids as $pdi) {
                    $bill_desc = BillDesc::select('gr_id','gr_number','ls_id','ls_number','selection_type','included_charges')->find($pdi);
                    if($bill_desc->selection_type == 'GR'){
                        $this->change_bill_status('GR', true, $bill_desc['gr_id'], $bill_desc->included_charges);
                    }
                    else if($bill_desc->selection_type == 'Loading Slip'){
                        $this->change_bill_status('Loading Slip',true,$bill_desc['ls_id'],'NONE');
                    }
                }
            }

            $cn_desc_ids = CreditNoteDesc::select('id')->where('cn_id', $pk)->pluck('id')->toArray();
            $cn_desc = CreditNoteDesc::whereIn('id', $cn_desc_ids)->delete();

            //FIND AND DELETE THE BRANCH
            $credit_note->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

            //DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'Credit Note';
            $t_data['t_against_id'] = $credit_note->id;
            $t_data['t_against_no'] = $credit_note->cn_number;
            $t_data['fin_year'] = $request->current_year;

            $this->delete_transaction_voucher($t_data);


            $this->createEditLog('deleted', 'credit_notes', $credit_note->id, $credit_note->cn_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = CreditNote::select('id', 'cn_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if(isset($request->note_name)){
            $query->where('note_name', $request->note_name);
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('cn_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $credit_notes = $query->get();
        return response(['credit_notes' => $credit_notes, 'draw' => $request->draw]);
    }

    public function bill_for_credit_note(Request $request){
        $branch_id = $request->branch_id;
        $acc_party_id = $request->party;
        $income_account_id = $request->income;
        $against = $request->against;


        if($against=='Bill'){
            // Initialize the query builder
            $query = Bill::select('id', 'bill_number')->where('fin_year',$request->current_year)->where('income', $income_account_id);
        }
        elseif($against=='Purchase Bill'){
            $query = Purchase::select('id', 'bill_number')->where('fin_year',$request->current_year);
        }

       

        $query->where('branch_id', $branch_id )->where('acc_party_id', $acc_party_id );

        if($request->exclude_bills)
        $query->whereNotIn('id',$request->exclude_bills);
        
        $query->whereDoesntHave('cn_desc');

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('bill_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $bills = $query->get();
        return response(['bills' => $bills, 'draw' => $request->draw]);
    }


    public function fetch_bill_details_for_credit_note(Request $request){
        $bill_id = $request->bill['id'];
        $against = $request->against;
        
        if($against=='Bill'){
            $bill = Bill::select('id','bill_number','bill_date','total_without_gst','other_plus','other_minus','total_without_rounding','igst_percent','cgst_percent','sgst_percent','total_igst_amount','total_cgst_amount','total_sgst_amount')->find($bill_id);
        }
        elseif($against=='Purchase Bill'){
            $bill = Purchase::select('id','bill_number','bill_date','total_without_gst','total_without_rounding','total_igst_amount','total_cgst_amount','total_sgst_amount')->with(['purchase_desc','purchase_desc.dr_account:id,account_name'])->find($bill_id);

            $igst_percent = 0;
            $cgst_percent = 0;
            $sgst_percent = 0;

            foreach($bill->purchase_desc as $pd){
                $igst_percent += $pd->igst_percent;
                $cgst_percent += $pd->cgst_percent;
                $sgst_percent += $pd->sgst_percent;
            }

            $bill['igst_percent'] = $igst_percent;
            $bill['cgst_percent'] = $cgst_percent;
            $bill['sgst_percent'] = $sgst_percent;

            $bill['expense'] = $pd['dr_account'];

        }

        return response()->json(['bill' => $bill]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'CreditNote');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = CreditNote::with([
                'branch:id,branch_name,branch_code',
                'acc_party:id,account_name',
                'cn_desc'
                ])
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('cn_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = CreditNote::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $credit_notesQuery = CreditNote::select('id', 'cn_number')->where('fin_year',$request->current_year);
        
        if (!empty($request->user_branch_ids)) {
            $credit_notesQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        
        $credit_notes = $credit_notesQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['credit_notes' => $credit_notes]);
    }
    

    public function get_credit_note_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = CreditNote::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'cn_desc',
            'income:id,account_name'
        ]);

        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['cn_number_from']) && isset($report_data['cn_number_to'])){
            $query->where('cn_number','>=',$report_data['cn_number_from']['cn_number'])->where('cn_number','<=',$report_data['cn_number_to']['cn_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('cn_date','>=',$report_data['from_date'])->where('cn_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['note_name']) ){
            $query->where('note_name', $report_data['note_name']);
        }


        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }

        


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('cn_number', 'like', '%'. $searchValue . '%');
            });
        }
        
        
        

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    public function get_next_cn_number(Request $request){
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        $note_name = $request->note_name;

        $abb = $note_name=='Credit Note' ? 'CN' : 'DN';

        $max = CreditNote::where('branch_id',$branch_id)->where('note_name',$note_name)->max('max_number');
        $credit_note_number = $branch_code.'-'.$abb.'-'.sprintf('%04d', $max+1);
        return response()->json(['cn_number' => strtoupper($credit_note_number), 'max_number' => $max+1]);
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $credit_note = CreditNote::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'cn_desc',
            'income:id,account_name'
        ])->find($pk);

        $credit_note->cn_desc->transform(function ($desc) { 
            return [
                'cn_desc_id' => $desc->id,
                'cn_id' => $desc->cn_id,
                'against' => $desc->against,
                'bill_id' => $desc->bill_id,
                'bill' => ['id'=>$desc->bill_id, 'bill_number'=>$desc->bill_number],
                'bill_date' => $desc->bill_date,
                'gr_number' => $desc->gr_number,
                'amount' => $desc->amount,
                'net_amount' => $desc->net_amount,
                'igst_percent' => $desc->igst_percent,
                'cgst_percent' => $desc->cgst_percent,
                'sgst_percent' => $desc->sgst_percent,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
            ];
        });


        return ['credit_note' =>$credit_note];
    }


    public function get_custom_fields(Request $request){
        $credit_note = CreditNote::select($request->custom_fields)->with($request->with_relations)->find($request->cn_id);
        return response()->json(['credit_note' => $credit_note]);
    }

    protected function get_max_number_from_cn_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    public function change_bill_status($selection_type, $status , $pk, $included_charges){
        if($selection_type == 'GR'){
            $GR_UPDATE = BookGr::find($pk);
            if($included_charges=='Freight' || $included_charges=='All'){
            $GR_UPDATE->freight_bill = $status;
            }
            if($included_charges=='Way Expenses' || $included_charges=='All'){
            $GR_UPDATE->way_exp_bill = $status;
            }
            $GR_UPDATE->save();
        }
        else if($selection_type == 'Loading Slip'){
            $LS_UPDATE = LoadingSlip::find($pk);
            $LS_UPDATE->bill_status = $status;
            $LS_UPDATE->save();
        }
    }

    protected  function  record_transaction_in_accounts($request,$credit_note){
        $t_data = [];
        $t_data['t_against'] = 'Credit Note';
        $t_data['t_against_id'] = $credit_note->id;
        $t_data['t_against_no'] = $credit_note->cn_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->cn_date;
        $t_data['voucher_time'] = null;
        
        $t_data['t_against_narration'] = $request->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        

        // BILL FREIGHT INCOME ACCOUNT
        $income_account_id = $credit_note->income_id;

    
        $t_data['cancellable'] = 1;

        if($credit_note->note_name=='Credit Note'){
            if($credit_note->total_without_tax>0 && $credit_note->acc_party_id && $income_account_id){
                $t_data['voucher_accounting_details']=[];
                // DEBIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$income_account_id, 'nature'=>'dr' , 'dr_amount'=>$credit_note->total_without_tax, 'cr_amount'=>0 ];
                
    
                if($credit_note->cgst_amount > 0){
                    $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'dr' , 'dr_amount'=>$credit_note->cgst_amount, 'cr_amount'=>0 ];
                }
    
                if($credit_note->sgst_amount > 0){
                    $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'dr' , 'dr_amount'=>$credit_note->sgst_amount, 'cr_amount'=>0 ];
                }
    
                if($credit_note->igst_amount > 0){
                    $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'dr' , 'dr_amount'=>$credit_note->igst_amount, 'cr_amount'=>0 ];
                }
              
    
                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=> $credit_note->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$credit_note->grand_total ];
    
                
                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;
    
                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }
    
                
    
                // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);
    
                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }
    
                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr;
    
                $this->create_transaction_voucher( $t_data );
                
    
            }
        }
        else{
            $t_data['t_against'] = 'Debit Note';
            if($credit_note->total_without_tax>0 && $credit_note->acc_party_id && $income_account_id){
                $t_data['voucher_accounting_details']=[];
                // DEBIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$income_account_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$credit_note->total_without_tax ];
                
    
                if($credit_note->cgst_amount > 0){
                    $t_data['voucher_accounting_details'][] = ['account_id'=>9, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$credit_note->cgst_amount ];
                }
    
                if($credit_note->sgst_amount > 0){
                    $t_data['voucher_accounting_details'][] = ['account_id'=>10, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$credit_note->sgst_amount ];
                }
    
                if($credit_note->igst_amount > 0){
                    $t_data['voucher_accounting_details'][] = ['account_id'=>11, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$credit_note->igst_amount ];
                }
              
    
                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=> $credit_note->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$credit_note->grand_total, 'cr_amount'=>0 ];
    
                
                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;
    
                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }
    
                
    
                // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);
    
                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }
    
                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr;
    
                $this->create_transaction_voucher( $t_data );
                
    
            }
        }
        

    }

}
