<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\FleetOwner\DieselBill;
use App\Models\FleetOwner\DieselBillDesc;
use App\Models\FleetOwner\DieselReceipt;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DieselBillController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'DieselBill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $diesel_bill = DieselBill::with([
            'branch:id,branch_name,branch_code',
            'filling_station:id,account_name',
            'vehicle:id,vehicle_number',
            'db_desc',
            'db_desc.diesel_receipt:id,dr_number,booking_date,qty,amount,rate,cash_amount,driver_on_account,vehicle_id,filling_station',
            'db_desc.diesel_receipt.driver_on_account:id,account_name',
            'db_desc.diesel_receipt.vehicle:id,vehicle_number',
        ])->where('fin_year',$request->current_year)->get();

        $diesel_bill->db_desc->transform(function ($desc) use ($diesel_bill)  {  
            $copy_object = json_decode(json_encode($desc->diesel_receipt));
            return [
                'db_desc_id' => $desc->id,
                'dr_id' => $desc->dr_id,
                'dr_number' => $desc->dr_number,
                'booking_date' => $desc->diesel_receipt->booking_date,
                'party_name' => $desc->diesel_receipt->filling_station,
                'slip_no' => $desc->slip_no,
                'vehicle' => $desc->diesel_receipt->vehicle,
                'qty'=>$desc->diesel_receipt->qty,
                'rate'=>$desc->diesel_receipt->rate,
                'amount'=>$desc->diesel_receipt->amount,
                'driver_on_account'=>$copy_object->driver_on_account,
                'cash_amount'=>$desc->diesel_receipt->cash_amount,
                'amount_before_discount'=>$desc->amount_before_discount,
                'discount'=>$desc->discount,
                'amount_after_discount' => $desc->amount_after_discount,

            ];
        });

        return response()->json($diesel_bill);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $diesel_bill = DieselBill::with([
            'branch:id,branch_name,branch_code',
            'filling_station:id,account_name',
            'vehicle:id,vehicle_number',
            'db_desc',
            'db_desc.diesel_receipt:id,dr_number,booking_date,qty,amount,rate,cash_amount,driver_on_account,vehicle_id,filling_station',
            'db_desc.diesel_receipt.driver_on_account:id,account_name',
            'db_desc.diesel_receipt.vehicle:id,vehicle_number',
        ])->find($pk);

        $diesel_bill->db_desc->transform(function ($desc) use ($diesel_bill)  {  
            $copy_object = json_decode(json_encode($desc->diesel_receipt));
            return [
                'db_desc_id' => $desc->id,
                'dr_id' => $desc->dr_id,
                'dr_number' => $desc->dr_number,
                'slip_no' => $desc->slip_no,
                'booking_date' => $desc->diesel_receipt->booking_date,
                'party_name' => $desc->diesel_receipt->filling_station,
                'vehicle' => $desc->diesel_receipt->vehicle,
                'qty'=>$desc->diesel_receipt->qty,
                'rate'=>$desc->diesel_receipt->rate,
                'amount'=>$desc->diesel_receipt->amount,
                'driver_on_account'=>$copy_object->driver_on_account,
                'cash_amount'=>$desc->diesel_receipt->cash_amount,
                'amount_before_discount'=>$desc->amount_before_discount,
                'discount'=>$desc->discount,
                'amount_after_discount' => $desc->amount_after_discount,

            ];
        });


        return response()->json(['diesel_bill' =>$diesel_bill]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'DieselBill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'db_number' => 'required|unique:diesel_bills,db_number',
            'filling_station' => 'required',
        ], [
            'branch.required' => 'Required',
            'db_number.required' => 'Required',
            'filling_station.required' => 'Required',
            'db_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE COVER TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_db_number($request->db_number);
            
            
            // SAVING A NEW GR
            $diesel_bill = new DieselBill();
            $diesel_bill->branch_id = $request->branch['id'];
            $diesel_bill->max_number = $max_number;
            $diesel_bill->db_number = $request->db_number;
            $diesel_bill->invoice_number = $request->invoice_number;
            $diesel_bill->filling_station_id = $request->filling_station['id'];
            $diesel_bill->vehicle_id = $request->vehicle?$request->vehicle['id']:null;
            $diesel_bill->db_date = $request->db_date;
            $diesel_bill->from_date = $request->from_date;
            $diesel_bill->to_date = $request->to_date;
            $diesel_bill->discount_percent = $request->discount_percent;
            $diesel_bill->discount_amount = $request->discount_amount;
            $diesel_bill->total_qty = $request->total_qty;
            $diesel_bill->total_amount = $request->total_amount;
            $diesel_bill->total_cash_amount = $request->total_cash_amount;
            $diesel_bill->total_bill_amount = $request->total_bill_amount;
            $diesel_bill->total_amount_before_discount = $request->total_amount_before_discount;
            $diesel_bill->total_discount = $request->total_discount;
            $diesel_bill->shortage_and_excess = $request->shortage_and_excess;
            $diesel_bill->total_amount_after_discount = $request->total_amount_after_discount;

            $diesel_bill->fin_year = $request->current_year;

            $diesel_bill->created_by = $request->user->id;

            // Save the model to the database
            $diesel_bill->save();

            

            // SAVING PACKAGE DETAILS
            $diesel_bill_desc = $request->db_desc;
            foreach ($diesel_bill_desc as $cld) {
                
                $db_desc = new DieselBillDesc();

                $db_desc->db_id = $diesel_bill->id;
                $db_desc->db_number = $diesel_bill->db_number;
                $db_desc->dr_id = $cld['dr_id'];
                $db_desc->dr_number = $cld['dr_number'];
                $db_desc->slip_no = $cld['slip_no'];
                $db_desc->amount_before_discount = $cld['amount_before_discount'];
                $db_desc->discount = $cld['discount'];
                $db_desc->amount_after_discount = $cld['amount_after_discount'];
            
                $db_desc->save();

                $dsl_rec = DieselReceipt::find($cld['dr_id']);
                $dsl_rec->slip_number = $cld['slip_no'];
                $dsl_rec->save();
            }
            // PACKAGE DETAILS SAVED

           
            $this->record_transaction_in_accounts($request, $diesel_bill);

             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($diesel_bill->id);
            $this->createEditLog('created', 'diesel_bill', $diesel_bill->id, $diesel_bill->db_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$diesel_bill->id, 'db_number'=>$diesel_bill->db_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'DieselBill');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $diesel_bill = DieselBill::find($pk);


        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($diesel_bill->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'db_number' => 'required',
            'filling_station' => 'required',
        ], [
            'branch.required' => 'Required',
            'db_number.required' => 'Required',
            'filling_station.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            

            // SAVING A NEW COVER
            $diesel_bill->invoice_number = $request->invoice_number;
            $diesel_bill->db_date = $request->db_date;
            $diesel_bill->discount_percent = $request->discount_percent;
            $diesel_bill->discount_amount = $request->discount_amount;
            $diesel_bill->total_qty = $request->total_qty;
            $diesel_bill->total_amount = $request->total_amount;
            $diesel_bill->total_cash_amount = $request->total_cash_amount;
            $diesel_bill->total_bill_amount = $request->total_bill_amount;
            $diesel_bill->total_amount_before_discount = $request->total_amount_before_discount;
            $diesel_bill->total_discount = $request->total_discount;
            $diesel_bill->shortage_and_excess = $request->shortage_and_excess;
            $diesel_bill->total_amount_after_discount = $request->total_amount_after_discount;


            $diesel_bill->updated_by = $request->user->id;

            $diesel_bill->save();
            // NEW COVER SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_db_desc = $request->newly_added_db_desc;
            foreach ($newly_added_db_desc as $nald) {
                
                $db_desc = new DieselBillDesc();

                $db_desc->db_id = $diesel_bill->id;
                $db_desc->db_number = $diesel_bill->db_number;
                $db_desc->slip_no = $nald['slip_no'];
                $db_desc->dr_id = $nald['dr_id'];
                $db_desc->dr_number = $nald['dr_number'];
                $db_desc->amount_before_discount = $nald['amount_before_discount'];
                $db_desc->discount = $nald['discount'];
                $db_desc->amount_after_discount = $nald['amount_after_discount'];

                $db_desc->save();

                $dsl_rec = DieselReceipt::find($nald['dr_id']);
                $dsl_rec->slip_number = $nald['slip_no'];
                $dsl_rec->save();

            }
            // PACKAGE DETAILS SAVED

            // SAVING PACKAGE DETAILS
            $updated_db_desc = $request->updated_db_desc;
            foreach ($updated_db_desc as $udd) {
                
                $db_desc = DieselBillDesc::find($udd['db_desc_id']);
                $db_desc->discount = $udd['discount'];
                $db_desc->amount_after_discount = $udd['amount_after_discount'];

                $db_desc->save();
            }
            // PACKAGE DETAILS SAVED

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_db_desc = $request->deleted_db_desc;
            if( !empty($deleted_db_desc) ){
                foreach ($deleted_db_desc as $dcd) {
                    $cld = DieselBillDesc::find($dcd);
                    $cld->delete();

                    $t_data2['t_against'] = 'DIESEL RECEIPT';
                    $t_data2['t_against_id'] = $cld->dr_id;
                    $t_data2['t_against_no'] = $cld->dr_number;
                    $t_data2['fin_year'] = $request->current_year;
                    $this->set_transaction_status($t_data2, true);
                }
                    
            }
            //DELETED EXISTING PACKAGE DETAILS

            $this->record_transaction_in_accounts($request, $diesel_bill);

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($diesel_bill->id);
            $this->createEditLog('updated', 'diesel_bill', $diesel_bill->id, $diesel_bill->db_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'DieselBill');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $diesel_bill = DieselBill::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($diesel_bill->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            $diesel_bill_desc_ids = DieselBillDesc::select('id')->where('db_id', $pk)->pluck('id')->toArray();

            // DELETING CHALLAN DESCRIPTIONS
            foreach ($diesel_bill_desc_ids as $pdi) {
                // MAKE GR TRANSACTION INACTIVE
                $bill_desc = DieselBillDesc::find($pdi);
                
                $t_data2['t_against'] = 'DIESEL RECEIPT';
                $t_data2['t_against_id'] = $bill_desc->dr_id;
                $t_data2['t_against_no'] = $bill_desc->dr_number;
                $t_data2['fin_year'] = $request->current_year;
                $this->set_transaction_status($t_data2, true);
            }


            $diesel_bill_desc = DieselBillDesc::whereIn('id', $diesel_bill_desc_ids)->delete();
    
            //FIND AND DELETE THE BRANCH
            $diesel_bill->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY
  

            // DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'DIESEL BILL';
            $t_data['t_against_id'] = $diesel_bill->id;
            $t_data['t_against_no'] = $diesel_bill->db_number;
            $t_data['fin_year'] = $request->current_year;
            $this->delete_transaction_voucher($t_data);


            $this->createEditLog('deleted', 'diesel_bill', $diesel_bill->id, $diesel_bill->db_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = DieselBill::select('id', 'db_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('db_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $diesel_bill = $query->get();
        return response(['diesel_bill' => $diesel_bill, 'draw' => $request->draw]);
    }

    public function fetch_against_details(Request $request){
        $filling_station_id = $request->filling_station['id'];
        $vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        $data = [];
        

        // GET ALL BILLS AGAINST THE PARTY
        // IF IT HAS A credit_note against it, then add a special field 'has_credit_note' => true in my response
        $diesels = DieselReceipt::select('id','dr_number','booking_date','slip_number','qty','rate','amount','cash_amount','driver_on_account','vehicle_id','filling_station')
                ->with([
                    'vehicle:id,vehicle_number',
                    'driver_on_account:id,account_name',
                    'filling_station:id,account_name'
                ]) 
                ->where('filling_station', $filling_station_id)
                ->when($vehicle_id, function ($query, $vehicle_id) {
                    return $query->where('vehicle_id', $vehicle_id);
                })
                ->whereBetween('booking_date', [$from_date, $to_date])
                ->whereDoesntHave('diesel_bill_desc')
                ->get();

                
    
        foreach ($diesels as $diesel) {
            $copy_of_object = json_decode(json_encode($diesel));
            // Add the diesel data to the response array
            $data[] = [
                'db_desc_id' => null,
                'dr_id' => $diesel->id,
                'dr_number' => $diesel->dr_number,
                'booking_date' => $diesel->booking_date,
                'slip_no' => $diesel->slip_number,
                'total_amount' => $diesel->amount+$diesel->cash_amount,
                'party_name' => $copy_of_object->filling_station->account_name,
                'vehicle' => $copy_of_object->vehicle,
                'qty'=>$diesel->qty,
                'rate'=>$diesel->rate,
                'amount'=>$diesel->amount,
                'driver_on_account'=>$copy_of_object->driver_on_account,
                'cash_amount'=>$diesel->cash_amount,
                'amount_before_discount'=>$diesel->amount+$diesel->cash_amount,
                'discount'=>0,
                'amount_after_discount' => $diesel->amount+$diesel->cash_amount,
                'checked' => true,
            ];
        }

        return response()->json(['bills' => $data]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'DieselBill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = DieselBill::with([
                    'branch:id,branch_name,branch_code',
                    'filling_station:id,account_name',
                ])
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('db_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = DieselBill::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $diesel_billQuery = DieselBill::select('id', 'db_number')->where('fin_year',$request->current_year);
        
        if (!empty($request->user_branch_ids)) {
            $diesel_billQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        
        $diesel_bill = $diesel_billQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['diesel_bill' => $diesel_bill]);
    }
    


    public function get_next_db_number(Request $request){
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        $max = DieselBill::where('branch_id',$branch_id)->max('max_number');
        $diesel_db_number = $branch_code.'-'.sprintf('%06d', $max+1);
        return response()->json(['db_number' => strtoupper($diesel_db_number), 'max_number' => $max+1]);
        
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $diesel_bill = DieselBill::with([
            'branch:id,branch_name,branch_code',
            'filling_station:id,account_name',
            'vehicle:id,vehicle_number',
            'db_desc',
            'db_desc.diesel_receipt:id,dr_number,booking_date,qty,amount,rate,cash_amount,driver_on_account,vehicle_id,filling_station',
            'db_desc.diesel_receipt.driver_on_account:id,account_name',
            'db_desc.diesel_receipt.vehicle:id,vehicle_number',
        ])->find($pk);

        
        $diesel_bill->db_desc->transform(function ($desc) use ($diesel_bill)  {  
            $copy_object = json_decode(json_encode($desc->diesel_receipt));
            return [
                'db_desc_id' => $desc->id,
                'dr_id' => $desc->dr_id,
                'dr_number' => $desc->dr_number,
                'booking_date' => $desc->diesel_receipt->booking_date,
                'slip_no' => $desc->slip_no,
                'party_name' => $desc->diesel_receipt->filling_station,
                'vehicle' => $desc->diesel_receipt->vehicle,
                'qty'=>$desc->diesel_receipt->qty,
                'rate'=>$desc->diesel_receipt->rate,
                'amount'=>$desc->diesel_receipt->amount,
                'driver_on_account'=>$copy_object->driver_on_account,
                'cash_amount'=>$desc->diesel_receipt->cash_amount,
                'amount_before_discount'=>$desc->amount_before_discount,
                'discount'=>$desc->discount,
                'amount_after_discount' => $desc->amount_after_discount,

            ];
        });

        return ['diesel_bill' =>$diesel_bill];
    }


    public function get_custom_fields(Request $request){
        $diesel_bill = DieselBill::select($request->custom_fields)->with($request->with_relations)->find($request->lh_id);
        return response()->json(['diesel_bill' => $diesel_bill]);
    }

    protected function get_max_number_from_db_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function record_transaction_in_accounts($request, $diesel_bill){
        $t_data = [];
        $t_data['t_against'] = 'DIESEL BILL';
        $t_data['t_against_id'] = $diesel_bill->id;
        $t_data['t_against_no'] = $diesel_bill->db_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->db_date;
        $t_data['voucher_time'] = Carbon::now()->format('H:i:s');
        $t_data['t_against_narration'] = "Diesel Bill No: ".$diesel_bill->db_number."  Bill No: ".$diesel_bill->invoice_number;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        

        $debited_accounts = [];
        $credited_accounts = [];

        $t_data['cancellable'] = 0;
        $bill_desc_ids = DieselBillDesc::select('id')->where('db_id', $diesel_bill->id)->pluck('id')->toArray();    
        // DELETING CHALLAN DESCRIPTIONS
        foreach ($bill_desc_ids as $pdi) {
            // MAKE GR TRANSACTION INACTIVE
            $bill_desc = DieselBillDesc::find($pdi);
             
            $diesel_receipt_data = DieselReceipt::select('id','dr_number','booking_date','qty','amount','rate','cash_amount','dr_on_account','driver_on_account','vehicle_id','filling_station')->find($bill_desc->dr_id);
            
            $dr_on_account =  $diesel_receipt_data->dr_on_account;
            $driver_on_account =  $diesel_receipt_data->driver_on_account;
            $filling_station =  $diesel_receipt_data->filling_station;

            $diesel_amount = $diesel_receipt_data->amount;
            $cash_amount = $diesel_receipt_data->cash_amount;
              
            $debited_accounts[]= [
                'account_id' => $dr_on_account,
                'amount' =>$diesel_amount
            ];
            $debited_accounts[]= [
                'account_id' => $driver_on_account,
                'amount' =>$cash_amount
            ];

            
            $credited_accounts[]= [
                'account_id' => $filling_station,
                'amount' =>$diesel_amount + $cash_amount
            ];

            $t_data2['t_against'] = 'DIESEL RECEIPT';
            $t_data2['t_against_id'] = $diesel_receipt_data->id;
            $t_data2['t_against_no'] = $diesel_receipt_data->dr_number;
            $t_data2['fin_year'] = $request->current_year;
            $this->set_transaction_status($t_data2, false);
        }

    
        // Find sums of common accounts in both debited and credited accounts
        $debited_accounts = $this->sum_common_accounts($debited_accounts);
        $credited_accounts = $this->sum_common_accounts($credited_accounts);

        if($diesel_bill->total_discount>0){
            $discount_received_acc = $this->get_acc_id('DISCOUNT RECEIVED');
            if(!$discount_received_acc){
                dd("DISCOUNT RECEIVED ACCOUNT NOT FOUND");
            }

            $credited_accounts[0]['amount'] -= $diesel_bill->total_discount; 

            $credited_accounts[]= [
                'account_id' => $discount_received_acc ,
                'amount' =>$diesel_bill->total_discount
            ];
        }

        // SHORT AND EXCESS
        if($diesel_bill->shortage_and_excess!=0){
            $SHORT_AND_EXCESS_ACC = $this->get_acc_id('SHORT AND EXCESS');
            if(!$SHORT_AND_EXCESS_ACC){
                dd("SHORT AND EXCESS ACCOUNT NOT FOUND");
            }


            if($diesel_bill->shortage_and_excess>=0){
                $debited_accounts[]= [
                    'account_id' => $SHORT_AND_EXCESS_ACC ,
                    'amount' => $diesel_bill->shortage_and_excess
                ];
                $credited_accounts[0]['amount'] += $diesel_bill->shortage_and_excess; 

            }
            else if($diesel_bill->shortage_and_excess<0){

                // $credited_accounts[]= [
                //     'account_id' => $SHORT_AND_EXCESS_ACC ,
                //     'amount' => abs($diesel_bill->shortage_and_excess)
                // ];
                $credited_accounts[]= [
                    'account_id' => $SHORT_AND_EXCESS_ACC ,
                    'amount' => abs($diesel_bill->shortage_and_excess)
                ];
                $credited_accounts[0]['amount'] -= abs($diesel_bill->shortage_and_excess); 

            }           
        }

        //dd($debited_accounts,$credited_accounts);




        // Combine the sums of all the debited and credited accounts
        if($debited_accounts && $credited_accounts){
            $t_data['voucher_accounting_details']=[];

            foreach ($debited_accounts as $dr_acc) {
                if($dr_acc['account_id'] && $dr_acc['amount'])
                $t_data['voucher_accounting_details'][] = ['account_id'=>$dr_acc['account_id'], 'nature'=>'dr' , 'dr_amount'=>$dr_acc['amount'], 'cr_amount'=>0 ];
            }

            foreach ($credited_accounts as $cr_acc) {
                if($cr_acc['account_id'] && $cr_acc['amount'])
                $t_data['voucher_accounting_details'][] = ['account_id'=>$cr_acc['account_id'], 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$cr_acc['amount'] ];
            }
        }



        //CHECK TOTAL DR AND CR
        $total_dr = 0;
        $total_cr = 0;

        foreach ($t_data['voucher_accounting_details'] as $vad) {
            $total_dr += $vad['dr_amount'];
            $total_cr += $vad['cr_amount'];
        }

        // Round both $total_dr and $total_cr to two decimal places
        $total_dr_rounded = round(floatval($total_dr), 2);
        $total_cr_rounded = round(floatval($total_cr), 2);

        // Check if they are not equal
        if ($total_dr_rounded != $total_cr_rounded) {
            // Output the values for debugging
            dd($total_dr_rounded, $total_cr_rounded);
            
            // Return response with an error message
            return response()->json(['message' => 'Error In Accounting Entries'], 422);
        }


        $t_data['total_dr_amount'] = $total_dr;
        $t_data['total_cr_amount'] = $total_cr; 

        

        $this->create_transaction_voucher( $t_data );

    }

    protected function sum_common_accounts($array){
        $result = array();

        $set_of_used_account_id = [];

        foreach ($array as $sub) {
            // Find account_id and sum it
            $account_id = $sub['account_id'];

            // Loop in array and find sum of the account_id
            if(in_array($account_id, $set_of_used_account_id)){
                continue;
            }

            $sum = 0;
            foreach ($array as $sub2) {
                if ($sub2['account_id'] == $account_id) {
                    $sum += $sub2['amount'];
                }
            }


            $set_of_used_account_id[] = $account_id;


            $result[] = ['account_id' => $account_id, 'amount' => $sum, 'account_name'=>$this->get_acc_name($account_id)];
            
        }
        return $result;
    }


}
