<?php

namespace App\Http\Controllers\HR;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\HR\Attendance;
use App\Models\Master\Employee;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class AttendanceController extends Controller
{
    use CommonTrait;
    use PermissionsTrait;

    public function get_month_attendance(Request $request)
    {
        $month = $request->month;
        $branch = $request->branch? $request->branch['id'] :null;

        $year = $this->get_start_end_year($request->current_year);
        
        $financialYear = ($month >= 4) ? $year->start_year : $year->end_year;
        
        // Calculate the number of days in the month
        $daysInMonth = Carbon::createFromDate($financialYear, $month, 1)->daysInMonth;

        // Create an array to store the result
        $result = [
            'financialYear' => $financialYear,
            'month' => $month,
            'daysInMonth' => $daysInMonth,
            'days' => [],
        ];

        // Get the list of employees
        // Get the first and last day of the month
        $startDate = Carbon::createFromDate($financialYear, $month, 1)->startOfDay();
        $endDate = Carbon::createFromDate($financialYear, $month, $daysInMonth)->endOfDay();

        // Get the list of employees
        $employees = Employee::select('id', 'emp_name')
            ->with(['vehicles' => function ($query) {
                $query->wherePivot('active', true)
                    ->select('vehicles.id', 'vehicles.vehicle_number', 'vehicles.vehicle_type')
                    ->withPivot('id', 'auth_date');
            }])
            ->where('category', $request->category)
            ->where('branch_id', $branch)
            ->where(function ($query) use ($startDate, $endDate) {
                $query->where('joining_date', '<=', $endDate)
                    ->where(function ($query) use ($startDate) {
                        $query->where('resignation_date', '>=', $startDate)
                                ->orWhereNull('resignation_date');
                    });
            })
            ->get();
        

       // Loop through each day of the month
        for ($day = 1; $day <= $daysInMonth; $day++) {
            $date = Carbon::createFromDate($financialYear, $month, $day);

            // Add the day name (short form) to the result array
            $result['days'][] = $date->format('D'); // 'D' format gives the short form of the day (Mon, Tue, etc.)

        }


        // Loop through each employee
        foreach ($employees as $employee) {
            // Initialize the attendance array for the current employee
            $attendanceData = [
                'id' => $employee->id, // You can add any employee details you want to show on the calendar here
                'name' => $employee->emp_name,
                'vehicle' => $employee->vehicles
            ];

            // Loop through each day of the month
            for ($day = 1; $day <= $daysInMonth; $day++) {
                $date = Carbon::createFromDate($financialYear, $month, $day);

                // Get the attendance record for the current employee and day
                $attendance = Attendance::where('emp_id', $employee->id)
                    ->where('attend_date', $date->toDateString())
                    ->first();

                // Add the day and its status to the attendance array
                $attendanceData[$day] = $attendance ? $attendance->attendance : '.';

                $attendanceData['attendance_pic'][$day] = $attendance ? $attendance->attendance_pic : null;
                $attendanceData['attendance_pic2'][$day] = $attendance ? $attendance->attendance_pic2 : null;

                $attendanceData['entry'][$day] = $attendance ? $attendance->entry_dt_tm : null;
                $attendanceData['exit'][$day] = $attendance ? $attendance->exit_dt_tm : null;

                $attendanceData['location'][$day] = $attendance && $attendance->longitude ? ['location'=>['longitude'=>$attendance->longitude, 'latitude'=>$attendance->latitude]] : null;
                $attendanceData['location2'][$day] = $attendance && $attendance->longitude ? ['location'=>['longitude'=>$attendance->longitude2, 'latitude'=>$attendance->latitude2]] : null;
            }

            // Add the attendance array to the result
            $result['employees'][] = $attendanceData;
        }

        // You can return this data or modify it according to your needs
        return response()->json($result);
    }

    public function mark_attendance(Request $request)
    {
        $minMaxDate = $this->get_min_max_date($request->current_year);

        $employee = Employee::find($request->emp_id);

        // Check if the attendance date is within the employee's employment period
        $attendanceDate = Carbon::parse($request->date);
        if ($attendanceDate->lt($employee->joining_date) || ($employee->resignation_date && $attendanceDate->gt($employee->resignation_date))) {
            return response()->json([
                'success' => false,
                'message' => 'Attendance date is outside the employee\'s employment period',
            ]);
        }

        $attendances = Attendance::where('emp_id', $request->emp_id)
            ->where('attend_date', '>=', $minMaxDate->min_date)
            ->where('attend_date', '<=', $minMaxDate->max_date)
            ->whereIn('attendance', ['CL', 'PL', 'SL'])
            ->select('attendance', DB::raw('count(*) as count'))
            ->where('fin_year',$request->current_year)
            ->groupBy('attendance')
            ->get();

        $data = [];    
        foreach ($attendances as $attendance) {
            $data[strtolower($attendance->attendance)] = $attendance->count;
        }
        // If you want to ensure that keys 'cl', 'pl', and 'sl' are set even if there are no records for them, you can add the following lines:
        $data['cl'] = $data['cl'] ?? 0;
        $data['pl'] = $data['pl'] ?? 0;
        $data['sl'] = $data['sl'] ?? 0;

        $cl_bal = $employee->casual_leaves - $data['cl'];
        $pl_bal = $employee->earning_leaves - $data['pl'];
        $sl_bal = $employee->sick_leaves - $data['sl'];

        
        
        $attendance = Attendance::where('emp_id', $request->emp_id)
            ->where('attend_date', $request->date)
            ->where('fin_year',$request->current_year)
            ->first();

        $success_flag = true;
        $message = 'Attendance marked successfully';

        if ($attendance) {
            
            if( $request->attendance != '.' ){
                if($request->attendance == 'CL' && $cl_bal <= 0){
                    $success_flag = false;
                    $message = 'You have no casual leaves left';
                }
                if($request->attendance == 'PL' && $pl_bal <= 0){
                    $success_flag = false;
                    $message = 'You have no earning leaves left';
                }
                if($request->attendance == 'SL' && $sl_bal <= 0){
                    $success_flag = false;
                    $message = 'You have no sick leaves left';
                }

                if(!$success_flag){
                    $attendance->attendance = 'A';
                }
                else{
                    $attendance->attendance = $request->attendance;
                }

                if(isset($request->attendance_pic))
                $attendance->attendance_pic = $request->attendance_pic;

                if(isset($request->attendance_pic2))
                $attendance->attendance_pic2 = $request->attendance_pic2;


                if(isset($request->location) && !$attendance->attendance_pic2)
                {
                    $attendance->latitude = $request->location['latitude'];
                    $attendance->longitude = $request->location['longitude'];

                    $attendance->entry_dt_tm = Carbon::now();
                }
                else if (isset($request->location) && $attendance->attendance_pic2){
                    $attendance->latitude2 = $request->location['latitude'];
                    $attendance->longitude2 = $request->location['longitude'];

                    $attendance->exit_dt_tm = Carbon::now();
                }

                $attendance->updated_by = $request->user->id;
                $attendance->save();
            } else {
                $attendance->delete();
            }
            
        } else {

            if( $request->attendance != '.' ){
                if($request->attendance == 'CL' && $cl_bal <= 0){
                    $success_flag = false;
                    $message = 'You have no casual leaves left';
                }
                if($request->attendance == 'PL' && $pl_bal <= 0){
                    $success_flag = false;
                    $message = 'You have no earning leaves left';
                }
                if($request->attendance == 'SL' && $sl_bal <= 0){
                    $success_flag = false;
                    $message = 'You have no sick leaves left';
                }

                

                $attendance = new Attendance();
                $attendance->emp_id = $request->emp_id;
                $attendance->attend_date = $request->date;
                if(!$success_flag){
                    $attendance->attendance = 'A';
                }
                else{
                    $attendance->attendance = $request->attendance;
                }
                $attendance->fin_year = $request->current_year;

                if(isset($request->attendance_pic))
                $attendance->attendance_pic = $request->attendance_pic;

                if(isset($request->attendance_pic2))
                $attendance->attendance_pic2 = $request->attendance_pic2;


                if(isset($request->location) && !$attendance->attendance_pic2)
                {
                    $attendance->latitude = $request->location['latitude'];
                    $attendance->longitude = $request->location['longitude'];

                    $attendance->entry_dt_tm = Carbon::now();
                }
                else if (isset($request->location) && $attendance->attendance_pic2){
                    $attendance->latitude2 = $request->location['latitude'];
                    $attendance->longitude2 = $request->location['longitude'];

                    $attendance->exit_dt_tm = Carbon::now();
                }
                

                $attendance->created_by = $request->user->id;
                $attendance->save();
            }
            
        }

        return response()->json([
            'success' => $success_flag,
            'message' => $message,
        ]);
    }


    public function mark_attendance_for_all(Request $request)
    {
        $minMaxDate = $this->get_min_max_date($request->current_year);

        $data = $request->data;
        $marking = $request->marking;
        $date = $request->date;
        $day = $request->day;

        foreach ($data as $data_item) {
            $attendance = Attendance::where('emp_id', $data_item['id'])
            ->where('attend_date', $date)
            ->where('fin_year',$request->current_year)
            ->first();

            if ($attendance) {
            
                if( $request->attendance != '.' ){
                    $attendance->attendance = $marking;
                    $attendance->updated_by = $request->user->id;
                    $attendance->save();
                }
                else{
                    $attendance->delete();
                }
            }
            else{
                $attendance = new Attendance();
                $attendance->emp_id = $data_item['id'];
                $attendance->attend_date = $date;
                $attendance->attendance = $data_item[$day];
                $attendance->fin_year = $request->current_year;
                $attendance->created_by = $request->user->id;
                $attendance->save();
            }
        }

        return response()->json("Success");
    }


    public function get_attendance_report(Request $request){
        // If for a particular employee
        
        $employee = $request->report_data['employee']??null;        
        $category = $request->report_data['category'];
        $month = $request->report_data['month'];
        $branch_id = $request->report_data['branch']?$request->report_data['branch']['id']:null;

        $year = $this->get_start_end_year($request->current_year);
        $financialYear = ($month >= 4) ? $year->start_year : $year->end_year;
        // Calculate the number of days in the month
        // Calculate the start date of the month
        $startDate = Carbon::createFromDate($financialYear, $month, 1)->startOfMonth();

        // Calculate the end date of the month
        $endDate = Carbon::createFromDate($financialYear, $month, 1)->endOfMonth();

        // Optionally, you can format the dates as needed
        $startDateFormatted = $startDate->format('Y-m-d');
        $endDateFormatted = $endDate->format('Y-m-d');

        // Calculate the number of days in the month
        $daysInMonth = $startDate->daysInMonth;

        // Loop through each day between $startDate and $endDate
        $sundays = 0;
        for ($date = $startDate->copy(); $date->lte($endDate); $date->addDay()) {
            // Check if the current day is a Sunday (Carbon's dayOfWeek returns 0 for Sunday)
            if ($date->dayOfWeek === 0) {
                $sundays++;
            }
        }


        $query = Employee::select('id','emp_name','category','branch_id')->with('branch:id,branch_code')->where('category',$category)
                    ->where(function ($query) use ($startDate, $endDate) {
                        $query->where('joining_date', '<=', $endDate)
                            ->where(function ($query) use ($startDate) {
                                $query->where('resignation_date', '>=', $startDate)
                                        ->orWhereNull('resignation_date');
                            });
                    });
                    
        
        if($employee){
            $query = $query->where('id',$employee['id']);
        }

        if($branch_id){
            $query = $query->where('branch_id',$branch_id);
        }

        $employees = $query->get();


        $result_data = [];


        foreach ($employees as $employee) {

            $emp_id = $employee['id'];

            $attendance_info = Attendance::where('emp_id',$emp_id)
                               ->where('attend_date','>=', $startDateFormatted)
                               ->where('attend_date','<=', $endDateFormatted)
                               ->select('attendance', DB::raw('count(*) as count'))
                                ->groupBy('attendance')
                                ->where('fin_year',$request->current_year)
                                ->get();

                             
            $counter = [];    
            foreach ($attendance_info as $attendance) {
                $counter[strtolower($attendance->attendance)] = $attendance->count;
            };
                                              
            
                $data = [
                    'emp_id' => $employee['id'],
                    'emp_name' => $employee['emp_name'],
                    'branch' => $employee['branch']['branch_code']??'',
                    'category' => $employee['category'],
                    'total_working_days' => $daysInMonth,
                    'present' => $counter['p']??0,
                    'absent' => $counter['a']??0,
                    'leaves' => ($counter['cl'] ?? 0) + ($counter['pl'] ?? 0) + ($counter['sl'] ?? 0),
                    'holidays' => $counter['h']??0,
                    'sundays' => $sundays,
                ];


            $result_data[] = $data;    

        }

    
        return response()->json(['data'=>$result_data]);
    }
}
