<?php

namespace App\Http\Controllers\Inventory;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Inventory\ProductCategory;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class ProductCategoryController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'ProductCategory');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // FETCH AND SEND ALL
        $categories = ProductCategory::all();
        return response()->json($categories);
    }

    public function show(Request $request, $pk)
    {
        // FETCH AND SEND
        $product_categories = ProductCategory::find($pk);
        return response()->json(['product_category' => $product_categories]);
    }

    public function store(Request $request)
    {

        $test = $this->is_new_permitted($request->user->id, 'ProductCategory');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'product_category' => 'required|unique:product_categories,product_category',
        ], [
            'product_category.required' => 'Required',
            'product_category.unique' => 'Already Exist!',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A NEW ProductCategory
            $product_categories = new ProductCategory();
            $product_categories->product_category = $request->product_category;
            $product_categories->hsn = $request->hsn;
            $product_categories->created_by = $request->user->id;
            $product_categories->save();
            // NEW ProductCategory SAVED

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($product_categories->id);
            $this->createEditLog('created', 'product_categories', $product_categories->id, $product_categories->product_category, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        }
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'ProductCategory');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $product_categories = ProductCategory::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($product_categories->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // UPDATING THE ProductCategory
            $product_categories->product_category = $request->product_category;
            $product_categories->hsn = $request->hsn;
            $product_categories->updated_by = $request->user->id;
            $product_categories->save();
            // ProductCategory UPDATED


            $newVal = $this->get_data_for_obj($product_categories->id);
            $this->createEditLog('updated', 'product_categories', $product_categories->id, $product_categories->product_category, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            

            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'ProductCategory');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $product_categories = ProductCategory::find($pk);

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($product_categories->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        // FIND AND DELETE THE ProductCategory
        $product_categories->delete();

        $this->createEditLog('deleted', 'product_categories', $product_categories->id, $product_categories->product_category, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // WHILE SEARCHING THIS WILL RUN
        $fields = ['id', 'product_category'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $query = ProductCategory::select($fields);


        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('product_category', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit);
        // Retrieve the filtered 
        $categories = $query->get();
        return response(['product_categories' => $categories, 'draw' => $request->draw]);
    }

    public function datatable(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'ProductCategory');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = ProductCategory::orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('product_category', 'like', '%'. $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = ProductCategory::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        $product_categories = ProductCategory::find($pk);
        return ['product_category' => $product_categories];
    }
}
