<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\Master\Route;
use App\Models\Master\RouteStation;
use App\Models\Master\RouteVehicle;
use App\Models\Master\Station;
use App\Models\Master\Vehicle;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;
use Psy\Readline\Hoa\Console;

class RouteController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'RouteMaster');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $routes = Route::with(['stations' => function ($q) {
            $q->select('stations.id','station_name')->withPivot('id','time_taken','distance','fuel');
        }])->get();
        return response()->json($routes);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND ONE
        $route = Route::with([
            'stations' => function ($query) {
               $query->select('stations.id', 'station_name')->withPivot('id','time_taken','distance','fuel');
            },
            'vehicles' => function ($query) {
                $query->select('vehicles.id', 'vehicle_number','vehicle_type')->withPivot('id','auth_date');
            }
        ])->findOrFail($pk);
        
        $result = [
            'id' => $route->id,
            'route_name' => $route->route_name,
            'fuel' => $route->fuel,
            'charge' => $route->charge,
            'total_distance' => $route->total_distance,
            'total_time_taken' => $route->total_time_taken,
            'max_turns_of_vehicle'=> $route->max_turns_of_vehicle,
            'station_description' => $route->stations->map(function ($station) {
                return [
                    'id' => $station->pivot->id,
                    'station' => [
                        'id' => $station->id,
                        'station_name' => $station->station_name,         
                    ],
                    'time_taken' => $station->pivot->time_taken,
                    'distance' => $station->pivot->distance,
                    'fuel' => $station->pivot->fuel
                ];
            }),
            'vehicles_description' => $route->vehicles->map(function ($vehicle) {
                return [
                    'id' => $vehicle->pivot->id,
                    'vehicle' => [
                        'id' => $vehicle->id,
                        'vehicle_number' => $vehicle->vehicle_number,   
                        'vehicle_type' => $vehicle->vehicle_type      
                    ],
                    'auth_date' => $vehicle->pivot->auth_date,
                ];
            }),
        ];
        return response()->json(['route' => $result]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'RouteMaster');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'route_name' => 'required|unique:routes,route_name',
        ], [
            'route_name.required' => 'Required',
            'route_name.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // SAVING A NEW Route
            $route = new Route;
            $route->route_name = $request->route_name ;
            $route->fuel = $request->fuel ;
            $route->charge = $request->charge ;
            $route->total_distance = $request->total_distance ;
            $route->total_time_taken = $request->total_time_taken ;
            $route->max_turns_of_vehicle = $request->max_turns_of_vehicle ;
            $route->created_by = $request->user->id;
            $route->save(); 
            // NEW Route SAVED

            // SAVING Stations
            $stations_data = $request->stations;
            if (!empty($stations_data)) {
                foreach ($stations_data as $station) {
                    if(!$station['station']){
                       continue;
                    }
                    $rs = new RouteStation();
                    $rs->route_id = $route->id;
                    $rs->station_id = $station['station']['id'];
                    $rs->time_taken = $station['time_taken'];
                    $rs->distance = $station['distance'];
                    $rs->fuel = $station['fuel'];
                    $rs->save();
                }
            }
            // Stations SAVED

            //  SAVING VEHICLES
            $vehicles_data = $request->vehicles;
            if (!empty($vehicles_data)) {
                foreach ($vehicles_data as $vehicle) {
                    if(!$vehicle['vehicle']){
                        continue;
                    }

                    $vh = new RouteVehicle();
                    $vh->route_id = $route->id;
                    $vh->vehicle_id = $vehicle['vehicle']['id'];
                    $vh->auth_date = $vehicle['auth_date'];
                    $vh->save();
                }
            }
            // SAVING VEHICLES

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($route->id);
            $this->createEditLog('created', 'routes', $route->id, $route->route_name, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'RouteMaster');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $route = Route::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($route->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A NEW Route
            $route->fuel = $request->fuel ;
            $route->charge = $request->charge ;
            $route->total_distance = $request->total_distance ;
            $route->total_time_taken = $request->total_time_taken;
            $route->max_turns_of_vehicle = $request->max_turns_of_vehicle ;
            $route->updated_by = $request->user->id;
            $route->save();
            // NEW Route SAVED
            
            // SAVING NEWLY ADDED Stations
            $newly_added_stations = $request->newly_added_stations;
            if (!empty($newly_added_stations)) {
                foreach ($newly_added_stations as $nas) {
                    if(!$nas['station']){
                        continue;
                    }

                    $rs = new RouteStation();
                    $rs->route_id = $route->id;
                    $rs->station_id = $nas['station']['id'];
                    $rs->time_taken = $nas['time_taken'];
                    $rs->distance = $nas['distance'];
                    $rs->fuel = $nas['fuel'];
                    $rs->save();
                }
            }
            // NEWLY ADDED Stations SAVED

            // SAVING UPDATES ON EXISTING Stations
            $updated_stations = $request->updated_stations;
            if( !empty($updated_stations) ){
                foreach ($updated_stations as $us) {
                    if(!$us['station']){
                        continue;
                     }
                        $rs = RouteStation::find( $us['id'] );
                        $rs->station_id = $us['station']['id'];
                        $rs->time_taken = $us['time_taken'];
                        $rs->distance = $us['distance'];
                        $rs->fuel = $us['fuel'];
                        $rs->save();
                }
            }
            // UPDATES SAVED

            // DELETING EXISTING station
            $deleted_stations = $request->deleted_stations;
            if( !empty($deleted_stations) ){
                    $rs = RouteStation::whereIn('id', $deleted_stations);
                    $rs->delete();
            }
            //DELETED EXISTING station

            //  SAVING NEW VEHICLES
            $newly_added_vehicles = $request->newly_added_vehicles;
            if (!empty($newly_added_vehicles)) {
                foreach ($newly_added_vehicles as $nav) {
                    if(!$nav['vehicle']){
                        continue;
                    }
                    $vh = new RouteVehicle();
                    $vh->route_id = $route->id;
                    $vh->vehicle_id = $nav['vehicle']['id'];
                    $vh->auth_date = $nav['auth_date'];
                    $vh->save();
                }
            }
            // SAVING NEW VEHICLES

            // DELETING EXISTING VEHICLES
            $deleted_vehicles = $request->deleted_vehicles;
            if( !empty($deleted_vehicles) ){
                    $dv = RouteVehicle::whereIn('id', $deleted_vehicles);
                    $dv->delete();
            }
            //DELETED EXISTING VEHICLES

            $newVal = $this->get_data_for_obj($route->id);
            $this->createEditLog('updated', 'routes', $route->id, $route->route_name , $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'RouteMaster');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $route = Route::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($route->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


        //FIND AND DELETE THE Route
        $route->delete();
        // Stations WILL BE CASCADED AUTOMATICALLY

        $this->createEditLog('deleted', 'routes', $route->id, $route->route_name,$oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // WHILE SEARCHING THIS WILL RUN
        // WHILE SEARCHING THIS WILL RUN
        $query = Route::select('id', 'route_name');

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('route_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('route_name', 'asc');

        // Retrieve the filtered 
        $route = $query->get();
        return response(['route' => $route, 'draw' => $request->draw]);
    }

    public function autocomplete_for_vehicle(Request $request){
        // Initialize the query builder
        // Return vehicle that belongs to $request->route_id  
        
        // Route id is passed from the request, If not then return all vehicles from Vehicle Model
        if($request->route_id){
            $max_turns = Route::select('max_turns_of_vehicle')->find($request->route_id)['max_turns_of_vehicle'];

            $query = Vehicle::select('vehicles.id', 'vehicle_number', 'vehicle_type')
                            ->whereNotIn('vehicles.id', function($subquery) use ($max_turns) {
                                $subquery->select('vehicle_id')
                                        ->from('book_grs')
                                        ->groupBy('vehicle_id')
                                        ->havingRaw('COUNT(*) > ?', [$max_turns])
                                        ->whereNotExists(function($query) {
                                            $query->select('id')
                                                ->from('pod_descs')
                                                ->whereColumn('pod_descs.gr_id', 'book_grs.id');
                                        });
                            });                     
        }else{
            $query = Vehicle::select('id', 'vehicle_number', 'vehicle_type');
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('vehicle_number', 'like', '%'. $searchVariable . '%');
        }
        
        return response(['vehicle' => $query->get(), 'draw' => $request->draw]);
    }

    public function autocomplete_for_station(Request $request){
        // Initialize the query builder
        // Return station that belongs to $request->route_id  
        
        // Route id is passed from the request, If not then return all station from Station Model
        if($request->route_id){
            $query = Route::find($request->route_id)->stations()->select('stations.id', 'station_name')->distinct();
        }
        else{
            $query = Station::select('id', 'station_name')->distinct();
        }    

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('station_name', 'like', '%'. $searchVariable . '%');
        }
        
        return response(['station' => $query->get(), 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'RouteMaster');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Route::select('id', 'route_name')
                ->with(['stations' => function ($q) {
                    $q->select('branch_name')->withPivot([]);
                }]) 
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('route_name', 'like', $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Route::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_route_data(Request $request){
        $routeId = $request->route_id;
        $sourceId = $request->source_id;
        $destinationId = $request->destination_id;

        $linked_ls = $request->linked_ls;

        $route = Route::find($routeId);

        $routeData = RouteStation::where('route_id', $routeId)->get();

        $sourceIndex = null;
        $destinationIndex = null;
        $total_hours = 0;
        $total_fuel = 0;

        // Find the index of the source station
        foreach ($routeData as $index => $station) {
            if ($station->station_id == $sourceId) {
                $sourceIndex = $index;
                break;
            }
        }

        // If source station is found, find the index of the destination station after the source
        if ($sourceIndex !== null) {
            foreach ($routeData as $index => $station) {
                if ($index > $sourceIndex && $station->station_id == $destinationId) {
                    $destinationIndex = $index;
                    break;
                }
            }

            // If destination station is found, calculate total hours and fuel
            if ($destinationIndex !== null) {
                for ($i = $sourceIndex; $i < $destinationIndex; $i++) {
                    $total_hours += $routeData[$i+1]->time_taken;
                    $total_fuel += $routeData[$i+1]->fuel;
                }
            }
        }

        if($linked_ls){
            $ls = LoadingSlip::find($linked_ls);
            $route_fuel_linked = $ls->route_fuel;

            $total_route_fuel = $route->fuel;

            $total_fuel = $total_route_fuel - $route_fuel_linked;
        }
        

        


        return response()->json(['estimated_hours' => $total_hours, 'fuel' => $total_fuel]);

    }

    public function get_route_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;

        $query = Route::with([
            'stations' => function ($query) {
               $query->select('stations.id', 'station_name')->withPivot('id','time_taken','distance','fuel');
            },
            'vehicles' => function ($query) {
                $query->select('vehicles.id', 'vehicle_number','vehicle_type')->withPivot('id','auth_date');
            }
        ]);

        if(isset($report_data['vehicle'])){
            $query->whereHas('vehicles', function ($query) use ($report_data) {
                $query->where('vehicle_id', $report_data['vehicle']['id']);
            });
        }

        if(isset($report_data['station'])){
            $query->whereHas('stations', function ($query) use ($report_data) {
                $query->where('station_id', $report_data['station']['id']);
            });
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('route_name', 'like', $searchValue . '%');
            });
        }


        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND ONE
        $route = Route::with([
            'stations' => function ($query) {
               $query->select('stations.id', 'station_name')->withPivot('id','time_taken','distance','fuel');
            },
            'vehicles' => function ($query) {
                $query->select('vehicles.id', 'vehicle_number','vehicle_type')->withPivot('id','auth_date');
            }
        ])->findOrFail($pk);
        
        $result = [
            'id' => $route->id,
            'route_name' => $route->route_name,
            'fuel' => $route->fuel,
            'charge' => $route->charge,
            'total_distance' => $route->total_distance,
            'total_time_taken' => $route->total_time_taken,
            'max_turns_of_vehicle' => $route->max_turns_of_vehicle,
            'station_description' => $route->stations->map(function ($station) {
                return [
                    'id' => $station->pivot->id,
                    'station' => [
                        'id' => $station->id,
                        'station_name' => $station->station_name,         
                    ],
                    'time_taken' => $station->pivot->time_taken,
                    'distance' => $station->pivot->distance,
                    'fuel' => $station->pivot->fuel,
                ];
            }),
            'vehicles_description' => $route->vehicles->map(function ($vehicle) {
                return [
                    'id' => $vehicle->pivot->id,
                    'vehicle' => [
                        'id' => $vehicle->id,
                        'vehicle_number' => $vehicle->vehicle_number,   
                        'vehicle_type' => $vehicle->vehicle_type      
                    ],
                    'auth_date' => $vehicle->pivot->auth_date,
                ];
            }),
        ];
        return ['route' => $result];   
    }
}
