<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Account;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\Master\Employee;
use App\Models\Master\EmployeeVehicle;
use App\Models\Master\MasterNarration;
use App\Models\Master\Vehicle;
use App\Models\Master\VehicleTyres;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class VehicleController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'VehicleDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $vehicle = Vehicle::with(['party' => function ($query) {
                        $query->select('id', 'party_name','party_code');
                    },])->get();
        return response()->json($vehicle);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $current_year = $request->current_year;
        $vehicle = Vehicle::with(['party' => function ($query) {
            $query->select('id', 'party_name', 'party_code');
        }, 'employees' => function ($query) {
            $query->wherePivot('active', true)
                ->select('employees.id', 'employees.emp_name')->withPivot('id', 'auth_date');
        },
        'vehicle_tyres'=>function($query) use ($current_year){
            $query->where('fin_year',$current_year);
        },
        'vehicle_tyres.product:id,product_name'
        ])->find($pk);
        
        $data = $vehicle->toArray();

        $data['employees'] = $vehicle->employees->map(function ($employee) {
            return [
                'id' => $employee->pivot->id,
                'auth_date' => $employee->pivot->auth_date,
                'driver' => [
                    'id' => $employee->id,
                    'emp_name' => $employee->emp_name,
                ],
            ];
        });

        $balance = Balances::where('account_id', $vehicle->account_id)->where('fin_year',$request->current_year)->first();

        return response()->json(['vehicle'=>$data , 'balance'=>$balance]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'VehicleDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'vehicle_number' => 'required|unique:vehicles,vehicle_number'
        ], [
            'vehicle_number.required' => 'Required',
            'vehicle_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // SAVING A NEW VEHICLE
            $vehicle = new Vehicle();
            
            $vehicle->vehicle_number = $request->vehicle_number;
            $vehicle->vehicle_type = $request->vehicle_type;
            $vehicle->self_market = $request->self_market;
            $vehicle->account_head = $request->account_head;
            $vehicle->fuel_type = $request->fuel_type;
            $vehicle->model = $request->model;
            $vehicle->length = $request->length;
            $vehicle->breadth = $request->breadth;
            $vehicle->height = $request->height;
            $vehicle->capacity = $request->capacity;
            $vehicle->mileage = $request->mileage;
            $vehicle->service_km = $request->service_km;
            $vehicle->service_days = $request->service_days;
            $vehicle->chasis_number = $request->chasis_number;
            $vehicle->engine_number = $request->engine_number;
            
            $vehicle->permit_number = $request->permit_number;
            $vehicle->permit_exp_date = $request->permit_exp_date;
            
            $vehicle->five_years_permit_number = $request->five_years_permit_number;
            $vehicle->five_years_permit_due_date = $request->five_years_permit_due_date;
            
            $vehicle->insurance = $request->insurance;
            $vehicle->insurance_due_date = $request->insurance_due_date;
            
            $vehicle->road_tax = $request->road_tax;
            $vehicle->road_tax_due_date = $request->road_tax_due_date;
            
            $vehicle->fitness_certificate_number = $request->fitness_certificate_number;
            $vehicle->fitness_due_date = $request->fitness_due_date;
            
            $vehicle->reg_number = $request->reg_number;
            $vehicle->reg_due_date = $request->reg_due_date;
            
            $vehicle->pollution = $request->pollution;
            $vehicle->pollution_due_date = $request->pollution_due_date;
            
            $vehicle->owner_permit_img = $request->owner_permit_img;
            $vehicle->insurance_img = $request->insurance_img;
            
            $vehicle->road_tax_img = $request->road_tax_img;
            $vehicle->fitness_certificate_img = $request->fitness_certificate_img;
            $vehicle->rc_img = $request->rc_img;
            $vehicle->pollution_certificate_img = $request->pollution_certificate_img;

            $vehicle->number_of_tyres = $request->number_of_tyres;
             
            if($request->owner){
                $vehicle->owner_id = $request->owner['id'];
            }
            else{
                $vehicle->owner_id = null;
            }

            $grp = Group::where('group_name', $request->account_head)->first();
            
            $new_account = new Account();
            $new_account->group_id = $grp->id ;
            $new_account->account_name = $vehicle->vehicle_number;
            $new_account->account_type = 'Vehicle';
            $new_account->created_by = $request->user->id;
            $new_account->save();

            $bal = new Balances();
            $bal->account_id = $new_account->id;
            $bal->opening_bal_dr = 0;
            $bal->opening_bal_cr = 0;
            $bal->opening_bal_date = Carbon::now();
            $bal->fin_year = $request->current_year;
            $bal->save();


            $vehicle->account_id = $new_account->id;
            
            $vehicle->created_by = $request->user->id;
            $vehicle->save(); 
            // NEW VEHICLE SAVED

            if($request->product_serial_desc){
                foreach ($request->product_serial_desc as $psd) {
                    if( $psd['ref_number'] && $psd['position'] && $psd['product']){
                        $vehicle_tyres = new VehicleTyres();
                        $vehicle_tyres->ref_number = $psd['ref_number'];
                        $vehicle_tyres->position = $psd['position'];
                        $vehicle_tyres->vehicle_id = $vehicle->id;
                        $vehicle_tyres->vehicle_number = $vehicle->vehicle_number;
                        $vehicle_tyres->product_id = $psd['product']['id'];
                        $vehicle_tyres->life_km = $psd['life_km'];
                        $vehicle_tyres->reading = $psd['reading'];
                        $vehicle_tyres->fin_year = $request->current_year;
                        $vehicle_tyres->save();
                    }
                }
            }

            // If $request->newly_added_employee, then add the vehicle to the employee
            if($request->newly_added_employee){
                $driver_id = $request->newly_added_employee['driver']['id'];
                
                // Check if the employee is already assigned to any vehicle
                $check = Vehicle::select('vehicle_number', 'employee_id')->where('employee_id', $driver_id)->first();
                if($check){
                    return response()->json(['errors' => ['driver' => ['Already Assigned To Another Vehicle = '.$check['vehicle_number'] .' !']]],422);
                }

                $emp_vehicle = new EmployeeVehicle();
                $emp_vehicle->employee_id = $driver_id;
                $emp_vehicle->vehicle_id = $vehicle->id;
                $emp_vehicle->auth_date = $request->newly_added_employee['auth_date'];
                $emp_vehicle->active = 1;
                $emp_vehicle->save();

                $employee = Employee::find($driver_id);
                $employee->vehicle_id = $vehicle->id;
                $employee->save();

                $vehicle->employee_id = $driver_id;
                $vehicle->save();
            }

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($vehicle->id,$request->current_year);
            $this->createEditLog('created', 'vehicles', $vehicle->id, $vehicle->vehicle_number, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'VehicleDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $vehicle = Vehicle::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($vehicle->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk, $request->current_year);

            // SAVING A NEW VEHICLE
            $vehicle->vehicle_number = $request->vehicle_number;
            $vehicle->vehicle_type = $request->vehicle_type;
            $vehicle->self_market = $request->self_market;
            $vehicle->account_head = $request->account_head;
            $vehicle->fuel_type = $request->fuel_type;
            $vehicle->model = $request->model;
            $vehicle->length = $request->length;
            $vehicle->breadth = $request->breadth;
            $vehicle->height = $request->height;
            $vehicle->capacity = $request->capacity;
            $vehicle->mileage = $request->mileage;
            $vehicle->service_km = $request->service_km;
            $vehicle->service_days = $request->service_days;
            $vehicle->chasis_number = $request->chasis_number;
            $vehicle->engine_number = $request->engine_number;
            
            $vehicle->permit_number = $request->permit_number;
            $vehicle->permit_exp_date = $request->permit_exp_date;
            
            $vehicle->five_years_permit_number = $request->five_years_permit_number;
            $vehicle->five_years_permit_due_date = $request->five_years_permit_due_date;
            
            $vehicle->insurance = $request->insurance;
            $vehicle->insurance_due_date = $request->insurance_due_date;
            
            $vehicle->road_tax = $request->road_tax;
            $vehicle->road_tax_due_date = $request->road_tax_due_date;
            
            $vehicle->fitness_certificate_number = $request->fitness_certificate_number;
            $vehicle->fitness_due_date = $request->fitness_due_date;
            
            $vehicle->reg_number = $request->reg_number;
            $vehicle->reg_due_date = $request->reg_due_date;
            
            $vehicle->pollution = $request->pollution;
            $vehicle->pollution_due_date = $request->pollution_due_date;
            
            $vehicle->owner_permit_img = $request->owner_permit_img;
            $vehicle->insurance_img = $request->insurance_img;
            
            $vehicle->road_tax_img = $request->road_tax_img;
            $vehicle->fitness_certificate_img = $request->fitness_certificate_img;
            $vehicle->rc_img = $request->rc_img;
            $vehicle->pollution_certificate_img = $request->pollution_certificate_img;

            $vehicle->number_of_tyres = $request->number_of_tyres;
             
            if($request->owner){
                $vehicle->owner_id = $request->owner['id'];
            }
            else{
                $vehicle->owner_id = null;
            }   
      
            
            // IF $request->deleted_employee, then delete the vehicle from the employee
            if($request->deleted_employee){
                $emp_vehicle = EmployeeVehicle::find($request->deleted_employee['id']);
                $emp_vehicle->deauth_date = now();
                $emp_vehicle->active = false;
                $emp_vehicle->save();

                $employee = Employee::find($request->deleted_employee['driver']['id']);
                $employee->vehicle_id = null;
                $employee->save();

                $vehicle->employee_id = null;
            }
            // If $request->newly_added_employee, then add the vehicle to the employee
            if($request->newly_added_employee){
                $driver_id = $request->newly_added_employee['driver']['id'];
                
                // Check if the employee is already assigned to any vehicle
                $check = Vehicle::select('vehicle_number', 'employee_id')->where('employee_id', $driver_id)->first();
                if($check){
                    return response()->json(['errors' => ['driver' => ['Already Assigned To Another Vehicle = '.$check['vehicle_number'] .' !']]],422);
                }

                $emp_vehicle = new EmployeeVehicle();
                $emp_vehicle->employee_id = $driver_id;
                $emp_vehicle->vehicle_id = $vehicle->id;
                $emp_vehicle->auth_date = $request->newly_added_employee['auth_date'];
                $emp_vehicle->active = true;
                $emp_vehicle->save();

                $employee = Employee::find($driver_id);
                $employee->vehicle_id = $vehicle->id;
                $employee->save();

                $vehicle->employee_id = $driver_id;
            }
             
            $vehicle->updated_by = $request->user->id;
            $vehicle->save();
            // NEW VEHICLE SAVED
            
            $existing = VehicleTyres::where('vehicle_id',$vehicle->id)->where('fin_year',$request->current_year)->delete();
            if($request->product_serial_desc){
                foreach ($request->product_serial_desc as $psd) {
                    if( $psd['ref_number'] && $psd['position']  && $psd['product']){
                        $vehicle_tyres = new VehicleTyres();
                        $vehicle_tyres->ref_number = $psd['ref_number'];
                        $vehicle_tyres->position = $psd['position'];
                        $vehicle_tyres->vehicle_id = $vehicle->id;
                        $vehicle_tyres->vehicle_number = $vehicle->vehicle_number;
                        $vehicle_tyres->product_id = $psd['product']['id'];
                        $vehicle_tyres->life_km = $psd['life_km'];
                        $vehicle_tyres->reading = $psd['reading'];
                        $vehicle_tyres->fin_year = $request->current_year;
                        $vehicle_tyres->save();
                    }
                }
            }

            $grp = Group::where('group_name', $request->account_head)->first();

            $account = Account::find($vehicle->account_id);
            $account->group_id = $grp->id;
            $account->updated_by = $request->user->id;
            $account->save();

            $bal = Balances::where('account_id', $account->id)->where('fin_year', $request->current_year)->first();
            $bal->opening_bal_dr = $request->opening_bal_dr;
            $bal->opening_bal_cr = $request->opening_bal_cr;
            $bal->save();


            $newVal = $this->get_data_for_obj($vehicle->id, $request->current_year);
            $this->createEditLog('updated', 'vehicles', $vehicle->id, $vehicle->vehicle_number, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'VehicleDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $vehicle = Vehicle::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($vehicle->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk, $request->current_year);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


            $existing = VehicleTyres::where('vehicle_id',$vehicle->id)->delete();
            
        //FIND AND DELETE THE VEHICLE
        $vehicle->delete();

        $account = Account::where('id',$vehicle->account_id)->first();
        $balances = Balances::where('account_id', $account->id)->delete();
        $account->delete();

        $this->createEditLog('deleted', 'vehicles', $vehicle->id, $vehicle->vehicle_number, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $fields = ['id', 'vehicle_number', 'vehicle_type'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $query = Vehicle::select($fields);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }

        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('vehicle_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('vehicle_number', 'asc');

        // Retrieve the filtered vehicles
        $vehicles = $query->get();

        return response(['vehicle' => $vehicles, 'draw' => $request->draw]);
    }

    public function get_autorised_driver($pk){
        // Initialize the query builder
        $vehicle = Vehicle::select('id', 'vehicle_number', 'vehicle_type')
                ->with([
                    'employees' => function ($query) {
                        $query->wherePivot('active', true)
                            ->select('employees.id', 'employees.emp_name')->withPivot('id', 'auth_date');
                    }
                ])
                ->find($pk);

        return response(['vehicle' => $vehicle]);
    }

    public function get_vehicle_narration_reminders(Request $request){
        $vehicle_narration = MasterNarration::with(['vehicle:id,vehicle_number','party:id,party_name','driver:id,emp_name'])
                        ->when($request->grlr, function ($query) use ($request) {
                            $query->where('against', $request->grlr);
                        })
                       ->where('alert_date', '<=', Carbon::now())->where('alert_on',1)->get();

        return response()->json(['reminders'=>$vehicle_narration]);

    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'VehicleDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Vehicle::select('id', 'vehicle_number')->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('vehicle_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_some_details(Request $request, $pk){

        $fields = ['id', 'vehicle_number', 'vehicle_type', 'owner_id'];

        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $vehicle = Vehicle::select($fields)->with(['party:id,party_name,account_id','party.account:id,account_name'])->find($pk);
        return response()->json($vehicle);
    }

    public function get_recent_gr_of_vehicle(Request $request, $pk){
        $book_gr = null;
        if($request->against=='GR'){
          $book_gr = BookGr::select('id','gr_number as against_number')->where('is_cancelled',false)->where('vehicle_id', $pk)->orderBy('id', 'desc')->first();

        }
        if($request->against=='Loading Slip'){
            $book_gr = LoadingSlip::select('id','ls_number as against_number')->where('vehicle_id', $pk)->orderBy('id', 'desc')->first();

        }


       $vehicle = Vehicle::select('id','vehicle_number','owner_id')->with(['party:id,party_name,account_id','party.account:id,account_name'])->find($pk);

       $account = null;
       if($vehicle->party){
         $account = $vehicle->party->account;
       }

       return response()->json(['gr'=>$book_gr, 'party_account'=>$account]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Vehicle::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    

    public function get_vehicle_reminders(Request $request)
    {
        $days = $request->days;
        // Calculate the date 15 days from now
        $allowedTillDate = Carbon::now()->addDays($days);

        // Define the expiry date fields and their corresponding document names
        $expiryDateFields = [
            'permit_exp_date' => 'Permit',
            'five_years_permit_due_date' => '5 Years Permit',
            'insurance_due_date' => 'Insurance',
            'road_tax_due_date' => 'Road Tax',
            'fitness_due_date' => 'Fitness Certificate',
            'pollution_due_date' => 'Pollution Certificate',
        ];

        $expiryFields = [
            'permit_exp_date' => 'permit_number',
            'five_years_permit_due_date' => 'five_years_permit_number',
            'insurance_due_date' => 'insurance',
            'road_tax_due_date' => 'road_tax',
            'fitness_due_date' => 'fitness_certificate_number',
            'pollution_due_date' => 'pollution',
        ];

        // Retrieve vehicles with expiry dates within the next 15 days
        $reminders = Vehicle::select(
            'vehicle_number',
            'id',
            ...array_keys($expiryDateFields), // Include all expiry date fields
            ...array_values($expiryFields) // Include all expiry fields
        )
        ->where(function ($query) use ($allowedTillDate, $expiryDateFields) {
            foreach ($expiryDateFields as $field => $documentName) {
                $query->orWhereDate($field, '<=', $allowedTillDate);
            }
        })
        ->get();

        // Group the data by expiry date fields
        $groupedData = [];

        foreach ($expiryDateFields as $field => $documentName) {
            $groupedData[$documentName] = [];

            $reminders->each(function ($reminder) use (&$groupedData, $field,$days, $documentName,$expiryFields, $allowedTillDate) {
                if ($reminder->{$field}) {
                    $dateValue = Carbon::parse($reminder->{$field});
                    $formattedDate = $dateValue->format('d M Y');

                    $currentDate = Carbon::now();
                    $difference = $dateValue->isPast() ? -$currentDate->diffInDays($dateValue) : $currentDate->diffInDays($dateValue) + 1;

                    if ($difference <= $days) {
                        $groupedData[$documentName][] = [
                            'vehicle_id' => $reminder->id,
                            'vehicle_number' => $reminder->vehicle_number,
                            'expiry_date' => $formattedDate,
                            'against' => $reminder->{$expiryFields[$field]},
                            'field_difference' => $difference,
                        ];
                    }
                }
            });
        }

        return response()->json($groupedData);
    }


    public function get_vehicle_diesel_reminders(Request $request)
    {
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        // Retrieve diesel receipts within the date range
        $diesel_receipts = DieselReceipt::whereBetween('booking_date', [$from_date, $to_date])
            ->select('vehicle_id', DB::raw('SUM(amount) as total_amount'), DB::raw('SUM(qty) as total_qty'))
            ->groupBy('vehicle_id')
            ->get();

        // Retrieve vehicles related to the diesel receipts
        $vehicles = Vehicle::whereIn('id', $diesel_receipts->pluck('vehicle_id'))->get();

        // Prepare the response data
        $groupedData = $vehicles->map(function ($vehicle) use ($diesel_receipts) {
            $receipt = $diesel_receipts->firstWhere('vehicle_id', $vehicle->id);
            return [
                'vehicle' => [
                    'vehicle_number' => $vehicle->vehicle_number,
                    'id' => $vehicle->id,
                ],
                'diesel_qty' => $receipt ? $receipt->total_qty : 0,
                'diesel_amount' => $receipt ? $receipt->total_amount : 0,
            ];
        });

        return response()->json($groupedData);
    }

    public function get_free_vehicle_reminders(Request $request)
    {
        $days = $request->days;
        // Calculate the date 15 days from now
        $allowedTillDate = Carbon::now()->subDays($days);

        $vehicles = Vehicle::select('id','vehicle_number','owner_id')->where('self_market','SELF')->get();

        // FETCHING THE VEHICLES THAT HAVE NO BOOKING BookGr in last $days
        $free_vehicles = $vehicles->filter(function ($vehicle) use ($allowedTillDate) {
            $book_gr = BookGr::where('vehicle_id', $vehicle->id)->where('is_cancelled',false)->where('gr_date','>=',$allowedTillDate)->first();
            $loading_slip = LoadingSlip::where('vehicle_id', $vehicle->id)->where('ls_date','>=',$allowedTillDate)->first();
            return (!$book_gr && !$loading_slip);
        });

        $reminder_data = [];

        $current_month = Carbon::now()->startOfMonth();
        $end_of_month = Carbon::now()->endOfMonth();



        foreach ($free_vehicles as $vehicle) {

            $total_bookings_this_month_ls = LoadingSlip::where('vehicle_id', $vehicle->id)
                                        ->whereBetween('ls_date', [$current_month, $end_of_month])
                                        ->count();
            $total_bookings_this_month_gr = BookGr::where('vehicle_id', $vehicle->id)
                                        ->whereBetween('gr_date', [$current_month, $end_of_month])
                                        ->count();


            $total_bookings_this_month = $total_bookings_this_month_ls + $total_bookings_this_month_gr;
            
            $last_booking_LS = LoadingSlip::select('id', 'ls_number as against_number', 'ls_date as against_date')
                ->where('vehicle_id', $vehicle->id)
                ->orderBy('ls_date', 'desc')
                ->first();

            $last_booking_GR = BookGr::select('id', 'gr_number as against_number', 'gr_date as against_date')
                ->where('vehicle_id', $vehicle->id)
                ->orderBy('gr_date', 'desc')
                ->first();

            $last_booking = null;
            $last_booking_type = '';
            if($last_booking_GR && $last_booking_LS){
                if($last_booking_GR->against_date >= $last_booking_LS->against_date){
                    $last_booking = $last_booking_GR;
                    $last_booking_type = 'GR';
                }
                else{
                    $last_booking = $last_booking_LS;
                    $last_booking_type = 'Loading Slip';
                }
            }
            elseif ($last_booking_GR) {
                $last_booking = $last_booking_GR;
                $last_booking_type = 'GR';
            } elseif ($last_booking_LS) {
                $last_booking = $last_booking_LS;
                $last_booking_type = 'Loading Slip';
            }
            

            $last_booking_date = $last_booking ? Carbon::parse($last_booking->against_date) : null;

            $reminder_data[] = [
                'vehicle_id' => $vehicle->id,
                'vehicle_number' => $vehicle->vehicle_number,
                'owner_id' => $vehicle->owner_id,
                'last_booking' => $last_booking,
                'field_difference' => $last_booking_date ? $last_booking_date->diffInDays(Carbon::now()) : null,
                'last_booking_type' => $last_booking_type,
                'total_bookings_this_month' => $total_bookings_this_month
            ];
        }

        return response()->json(['reminders'=>$reminder_data]);


    }


    public function get_vehicle_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;

        $query = Vehicle::select('id','vehicle_number','vehicle_type','owner_id','permit_number','five_years_permit_number','insurance','road_tax','fitness_certificate_number','reg_number','pollution','permit_exp_date','five_years_permit_due_date','insurance_due_date','road_tax_due_date','fitness_due_date','reg_due_date','pollution_due_date')->with('party:id,party_name,mob_number,pan_number');

        if(isset($report_data['vehicle'])){
            $query->where('id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['vehicle_type'])){
            $query->where('vehicle_type', $report_data['vehicle_type']);
        }

        if(isset($report_data['owner'])){
            $query->where('owner_id', $report_data['owner']['id']);
        }

        if(isset($report_data['self_market'])){
            $query->where('self_market', $report_data['self_market']);
        }

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('vehicle_number', 'like', $searchValue . '%')
                    ->orWhere('vehicle_type', 'like', $searchValue . '%');
            });
        }


        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    protected function get_data_for_obj($pk, $current_year){
        $vehicle = Vehicle::with(['party' => function ($query) {
            $query->select('id', 'party_name', 'party_code');
        }, 'employees' => function ($query) {
            $query->wherePivot('active', true)
                ->select('employees.id', 'employees.emp_name')->withPivot('id', 'auth_date');
        },
        'vehicle_tyres'=>function($query) use ($current_year){
            $query->where('fin_year',$current_year);
        },
        'vehicle_tyres.product:id,product_name'
        ])->find($pk);
        
        $data = $vehicle->toArray();

        $data['employees'] = $vehicle->employees->map(function ($employee) {
            return [
                'id' => $employee->pivot->id,
                'auth_date' => $employee->pivot->auth_date,
                'driver' => [
                    'id' => $employee->id,
                    'emp_name' => $employee->emp_name,
                ],
            ];
        });

        $balance = Balances::where('account_id', $vehicle->account_id)->where('fin_year',$current_year)->first();

        return ['vehicle'=>$data, 'balance'=>$balance];
    }
    
     public function get_empty_vehicle_report(Request $request){


            $vehicles = '';
            $report = '';
            // dd($request->report_data['vehicle']['vehicle_number']);
            if(isset($request->report_data)){
                $report = $request->report_data;
                // dd($report);
                if(isset($report['vehicle']['vehicle_number']) && $report['vehicle']['vehicle_number'] != null){
                    $vehicles = Vehicle::where('self_market','SELF')->where('vehicle_number', $report['vehicle']['vehicle_number'])
                    ->get();
                }
                else{
                    $vehicles = Vehicle::where('self_market','SELF')
                    ->get();
                }
            }
            else{
                $vehicles = Vehicle::where('self_market','SELF')
                ->get();
            }

            $emptyVehicles = [

            ];

            $filter_status = $report['status'];

$emptyVehicles = [];

foreach ($vehicles as $vehicle) {
    $latestGr = $vehicle->gr()->latest()->first(); // Retrieve the latest GR
    $latestLS = $vehicle->loadingSlip()->latest()->first(); // Retrieve the latest Loading Slip
    $podDesc = $latestGr ? $latestGr->pod_desc()->first() : null;
    $pod = $podDesc ? $podDesc->pod : null; // Retrieve Pod if exists

    $GR_DATE = $latestGr ? $latestGr->gr_date : null;
    $LS_DATE = $latestLS ? $latestLS->ls_date : null;
    $POD_DATE = $pod ? $pod->delivery_date : null;

    $status = 'UNKNOWN'; // Default Status
    $isLatest = false; // Default assumption

    $lastDocumentType = null;
    $lastDocumentNumber = null;
    $lastDocumentId = null;

    if ($latestGr && (!$latestLS || $GR_DATE > $LS_DATE)) {
        // GR is the latest document
        $isLatest = true;
        $lastDocumentType = 'GR';
        $lastDocumentNumber = $latestGr->gr_number;
        $lastDocumentId = $latestGr->id;

        if ($pod) {
            $status = 'EMPTY'; // Delivered
        } else {
            $status = 'IN TRANSIT'; // Still in transit
        }

        $emptyVehicles[] = [
            'vehicle_number' => $vehicle->vehicle_number,
            'destination' => $latestGr->destination_name,
            'last_pod_number' => $pod ? $pod->pod_number : '',
            'status' => $status,
            'is_latest' => $isLatest,
            'last_document_type' => $lastDocumentType,
            'last_document_number' => $lastDocumentNumber,
            'last_document_id' => $lastDocumentId
        ];
    } elseif ($latestLS && (!$latestGr || $LS_DATE > $GR_DATE)) {
        // Loading Slip is the latest document
        $isLatest = true;
        $lastDocumentType = 'Loading Slip';
        $lastDocumentNumber = $latestLS->ls_number;
        $lastDocumentId = $latestLS->id;

        $status = $latestLS->unload_date ? 'EMPTY' : 'IN TRANSIT';

        $emptyVehicles[] = [
            'vehicle_number' => $vehicle->vehicle_number,
            'destination' => $latestLS->destination_name ?? '',
            'last_pod_number' => '',
            'status' => $status,
            'is_latest' => $isLatest,
            'last_document_type' => $lastDocumentType,
            'last_document_number' => $lastDocumentNumber,
            'last_document_id' => $lastDocumentId
        ];
    } else {
        // The vehicle might have outdated GR or LS but is not the latest document
        $isLatest = false;
    }
}

// Filter based on provided status
if ($filter_status) {
    $emptyVehicles = collect($emptyVehicles)->filter(function ($value) use ($filter_status) {
        return $value['status'] == $filter_status;
    })->values();
}

// Return the JSON response
return response()->json(['data' => $emptyVehicles, 'draw' => $request->draw]);

            
            
    }

}
