<?php

namespace App\Http\Middleware;

use App\Models\User;
use Closure;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;
use Tymon\JWTAuth\Exceptions\TokenInvalidException;
use Tymon\JWTAuth\Facades\JWTAuth;

class CustomJWTMiddleware
{
    public function handle($request, Closure $next)
    {
        try {
            // Parse the JWT token from the request header
            $parsed_token = JWTAuth::parseToken();
            
            // Check if company and current year are already cached
            $company = Cache::remember('jwt_claim_company_' . $request->bearerToken(), 60, function () use ($parsed_token) {
                return $parsed_token->getClaim('company');
            });

            $current_year = Cache::remember('jwt_claim_current_year_' . $request->bearerToken(), 60, function () use ($parsed_token) {
                return $parsed_token->getClaim('current_year');
            });

            $user_branch_ids = Cache::remember('jwt_claim_user_branch_ids_' . $request->bearerToken(), 60, function () use ($parsed_token) {
                return $parsed_token->getClaim('user_branch_ids');
            });

            try {
                // Update the 'database' config in the 'dynamic' connection to point to the current company's database
                config(['database.connections.dynamic.database' => config('app.SERVER_PREFIX') .strtolower($company['company_code'])]);
            
                // Reconnect to the updated database connection
                DB::connection('dynamic')->reconnect();
            
                // Set the default database connection to 'dynamic'
                DB::setDefaultConnection('dynamic');
    
            } catch (\Exception $e) {
                // Handle the exception here, for example by returning a 500 error response
                return response()->json(['message' => 'Failed to update dynamic database connection'], 500);
            }

            // Retrieve the user associated with the JWT token
            $token = Cache::remember('jwt_token_' . $request->bearerToken(), 60, function () use ($parsed_token) {
                return User::where('username', $parsed_token->getPayload()->get('sub'))->firstOrFail();
            });

            // If the user, company, or current year is missing, return a 401 Unauthorized error response
            if (!$token || !$company || !$current_year) {
                return response()->json(['message' => 'Unauthorized User'], 401);
            }

            $request->merge(['user' => $token, 'user_branch_ids'=>$user_branch_ids, 'company' => $company, 'current_year' => $current_year]);
            
            // Call the next middleware in the pipeline
            return $next($request);
            
        } catch (TokenInvalidException $e) {
            // If the token is invalid, return a 401 Unauthorized error response
            return response()->json(['message' => 'Invalid token'], 401);
        }
    }
}